"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
from . import common_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _RadiensDatasourceType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _RadiensDatasourceTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_RadiensDatasourceType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    ANY: _RadiensDatasourceType.ValueType  # 0
    TIMESERIES: _RadiensDatasourceType.ValueType  # 1

class RadiensDatasourceType(_RadiensDatasourceType, metaclass=_RadiensDatasourceTypeEnumTypeWrapper):
    """used by sapiensserver client"""

ANY: RadiensDatasourceType.ValueType  # 0
TIMESERIES: RadiensDatasourceType.ValueType  # 1
global___RadiensDatasourceType = RadiensDatasourceType

@typing.final
class DataSourceRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DSOURCEID_FIELD_NUMBER: builtins.int
    RENAMEDSRCID_FIELD_NUMBER: builtins.int
    renameDsrcID: builtins.str
    @property
    def dsourceID(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        dsourceID: collections.abc.Iterable[builtins.str] | None = ...,
        renameDsrcID: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["renameDsrcID", b"renameDsrcID"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dsourceID", b"dsourceID", "renameDsrcID", b"renameDsrcID"]) -> None: ...

global___DataSourceRequest = DataSourceRequest

@typing.final
class DataSourceSetSaveRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PATH_FIELD_NUMBER: builtins.int
    BASENAME_FIELD_NUMBER: builtins.int
    DSOURCEID_FIELD_NUMBER: builtins.int
    FILETYPE_FIELD_NUMBER: builtins.int
    ISFORCE_FIELD_NUMBER: builtins.int
    ISBACKGROUNDKPI_FIELD_NUMBER: builtins.int
    PARENTDSOURCEID_FIELD_NUMBER: builtins.int
    ALLOWGPIOONLYXDAT_FIELD_NUMBER: builtins.int
    FINDASSOCIATED_FIELD_NUMBER: builtins.int
    path: builtins.str
    baseName: builtins.str
    dsourceID: builtins.str
    """datasource manager data source ID"""
    fileType: common_pb2.RadixFileTypes.ValueType
    """optional string uID = 4;  // deprecated"""
    isForce: builtins.bool
    isBackgroundKPI: builtins.bool
    parentDsourceID: builtins.str
    allowGPIOOnlyXDAT: builtins.bool
    """optional bool forceNoSorter = 9; // deprecated
    if allowGPIOOnlyXDAT is set, then an XDAT file with only GPIO data can be opened/registered. 
    This option was created for use with Curate to slice GPIO channels from the XDAT sidecar that gets saved alongside the spikes in spikes-only mode
    If allowGPIOOnlyXDAT is no set or false, then XDAT files with no AMP channels are considered proxies for their associated spikes files,
    which will be opened instead.
    """
    findAssociated: builtins.bool
    """if true, find and return associated datasources as well"""
    def __init__(
        self,
        *,
        path: builtins.str | None = ...,
        baseName: builtins.str | None = ...,
        dsourceID: builtins.str | None = ...,
        fileType: common_pb2.RadixFileTypes.ValueType | None = ...,
        isForce: builtins.bool | None = ...,
        isBackgroundKPI: builtins.bool | None = ...,
        parentDsourceID: builtins.str | None = ...,
        allowGPIOOnlyXDAT: builtins.bool | None = ...,
        findAssociated: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["allowGPIOOnlyXDAT", b"allowGPIOOnlyXDAT", "baseName", b"baseName", "dsourceID", b"dsourceID", "fileType", b"fileType", "findAssociated", b"findAssociated", "isBackgroundKPI", b"isBackgroundKPI", "isForce", b"isForce", "parentDsourceID", b"parentDsourceID", "path", b"path"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["allowGPIOOnlyXDAT", b"allowGPIOOnlyXDAT", "baseName", b"baseName", "dsourceID", b"dsourceID", "fileType", b"fileType", "findAssociated", b"findAssociated", "isBackgroundKPI", b"isBackgroundKPI", "isForce", b"isForce", "parentDsourceID", b"parentDsourceID", "path", b"path"]) -> None: ...

global___DataSourceSetSaveRequest = DataSourceSetSaveRequest

@typing.final
class DataSourceIDReply(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SORTEDID_FIELD_NUMBER: builtins.int
    @property
    def sortedID(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        sortedID: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["sortedID", b"sortedID"]) -> None: ...

global___DataSourceIDReply = DataSourceIDReply

@typing.final
class FileDescriptors(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DESC_FIELD_NUMBER: builtins.int
    @property
    def desc(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[common_pb2.FileDescriptor]: ...
    def __init__(
        self,
        *,
        desc: collections.abc.Iterable[common_pb2.FileDescriptor] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["desc", b"desc"]) -> None: ...

global___FileDescriptors = FileDescriptors

@typing.final
class DataSourceSetSaveReply(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DSOURCEID_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    ASSOCIATEDDSRCS_FIELD_NUMBER: builtins.int
    dsourceID: builtins.str
    """datasource manager data source ID"""
    @property
    def status(self) -> common_pb2.DataSourceStatus: ...
    @property
    def associatedDsrcs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[common_pb2.FileDescriptor]: ...
    def __init__(
        self,
        *,
        dsourceID: builtins.str | None = ...,
        status: common_pb2.DataSourceStatus | None = ...,
        associatedDsrcs: collections.abc.Iterable[common_pb2.FileDescriptor] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["dsourceID", b"dsourceID", "status", b"status"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["associatedDsrcs", b"associatedDsrcs", "dsourceID", b"dsourceID", "status", b"status"]) -> None: ...

global___DataSourceSetSaveReply = DataSourceSetSaveReply

@typing.final
class DataSourceStatusMap(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class DsourceEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> common_pb2.DataSourceStatus: ...
        def __init__(
            self,
            *,
            key: builtins.str | None = ...,
            value: common_pb2.DataSourceStatus | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    DSOURCE_FIELD_NUMBER: builtins.int
    ORDEREDIDS_FIELD_NUMBER: builtins.int
    @property
    def dsource(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, common_pb2.DataSourceStatus]:
        """key: datasource ID"""

    @property
    def orderedIDs(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        dsource: collections.abc.Mapping[builtins.str, common_pb2.DataSourceStatus] | None = ...,
        orderedIDs: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["dsource", b"dsource", "orderedIDs", b"orderedIDs"]) -> None: ...

global___DataSourceStatusMap = DataSourceStatusMap

@typing.final
class CopyRemoveDataSourceFileRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MultipleFiles(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        SRC_FIELD_NUMBER: builtins.int
        DESTPATH_FIELD_NUMBER: builtins.int
        DESTFILETYPE_FIELD_NUMBER: builtins.int
        destPath: builtins.str
        destFileType: common_pb2.RadixFileTypes.ValueType
        @property
        def src(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[common_pb2.FileDescriptor]: ...
        def __init__(
            self,
            *,
            src: collections.abc.Iterable[common_pb2.FileDescriptor] | None = ...,
            destPath: builtins.str | None = ...,
            destFileType: common_pb2.RadixFileTypes.ValueType | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["destFileType", b"destFileType", "destPath", b"destPath"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["destFileType", b"destFileType", "destPath", b"destPath", "src", b"src"]) -> None: ...

    @typing.final
    class OneFile(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        SRC_FIELD_NUMBER: builtins.int
        DEST_FIELD_NUMBER: builtins.int
        @property
        def src(self) -> common_pb2.FileDescriptor: ...
        @property
        def dest(self) -> common_pb2.FileDescriptor: ...
        def __init__(
            self,
            *,
            src: common_pb2.FileDescriptor | None = ...,
            dest: common_pb2.FileDescriptor | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["dest", b"dest", "src", b"src"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["dest", b"dest", "src", b"src"]) -> None: ...

    MANY_FIELD_NUMBER: builtins.int
    ONE_FIELD_NUMBER: builtins.int
    ISFORCE_FIELD_NUMBER: builtins.int
    ISVALIDATE_FIELD_NUMBER: builtins.int
    isForce: builtins.bool
    isValidate: builtins.bool
    @property
    def many(self) -> global___CopyRemoveDataSourceFileRequest.MultipleFiles: ...
    @property
    def one(self) -> global___CopyRemoveDataSourceFileRequest.OneFile: ...
    def __init__(
        self,
        *,
        many: global___CopyRemoveDataSourceFileRequest.MultipleFiles | None = ...,
        one: global___CopyRemoveDataSourceFileRequest.OneFile | None = ...,
        isForce: builtins.bool | None = ...,
        isValidate: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["desc", b"desc", "isForce", b"isForce", "isValidate", b"isValidate", "many", b"many", "one", b"one"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["desc", b"desc", "isForce", b"isForce", "isValidate", b"isValidate", "many", b"many", "one", b"one"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["desc", b"desc"]) -> typing.Literal["many", "one"] | None: ...

global___CopyRemoveDataSourceFileRequest = CopyRemoveDataSourceFileRequest

@typing.final
class DataSourceFileSetStat(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NUMFILES_FIELD_NUMBER: builtins.int
    NUMBYTESPRIMARYDATA_FIELD_NUMBER: builtins.int
    NUMBYTESMETADATA_FIELD_NUMBER: builtins.int
    NUMBYTES_FIELD_NUMBER: builtins.int
    ISMETADATAFILE_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    NUMCHANNELS_FIELD_NUMBER: builtins.int
    DURATIONSEC_FIELD_NUMBER: builtins.int
    DATASETUID_FIELD_NUMBER: builtins.int
    DATASETCHECKSUM_FIELD_NUMBER: builtins.int
    DATASETPROVENANCE_FIELD_NUMBER: builtins.int
    SAMPLERATE_FIELD_NUMBER: builtins.int
    BACKGROUNDPROC_FIELD_NUMBER: builtins.int
    numFiles: builtins.int
    numBytesPrimaryData: builtins.int
    numBytesMetaData: builtins.int
    numBytes: builtins.int
    isMetadataFile: builtins.bool
    numChannels: builtins.int
    durationSec: builtins.float
    datasetUID: builtins.str
    datasetChecksum: builtins.str
    sampleRate: builtins.int
    @property
    def timeStamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def datasetProvenance(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def backgroundProc(self) -> global___BackgroundProcessStatus: ...
    def __init__(
        self,
        *,
        numFiles: builtins.int | None = ...,
        numBytesPrimaryData: builtins.int | None = ...,
        numBytesMetaData: builtins.int | None = ...,
        numBytes: builtins.int | None = ...,
        isMetadataFile: builtins.bool | None = ...,
        timeStamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        numChannels: builtins.int | None = ...,
        durationSec: builtins.float | None = ...,
        datasetUID: builtins.str | None = ...,
        datasetChecksum: builtins.str | None = ...,
        datasetProvenance: collections.abc.Iterable[builtins.str] | None = ...,
        sampleRate: builtins.int | None = ...,
        backgroundProc: global___BackgroundProcessStatus | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["backgroundProc", b"backgroundProc", "datasetChecksum", b"datasetChecksum", "datasetUID", b"datasetUID", "durationSec", b"durationSec", "isMetadataFile", b"isMetadataFile", "numBytes", b"numBytes", "numBytesMetaData", b"numBytesMetaData", "numBytesPrimaryData", b"numBytesPrimaryData", "numChannels", b"numChannels", "numFiles", b"numFiles", "sampleRate", b"sampleRate", "timeStamp", b"timeStamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["backgroundProc", b"backgroundProc", "datasetChecksum", b"datasetChecksum", "datasetProvenance", b"datasetProvenance", "datasetUID", b"datasetUID", "durationSec", b"durationSec", "isMetadataFile", b"isMetadataFile", "numBytes", b"numBytes", "numBytesMetaData", b"numBytesMetaData", "numBytesPrimaryData", b"numBytesPrimaryData", "numChannels", b"numChannels", "numFiles", b"numFiles", "sampleRate", b"sampleRate", "timeStamp", b"timeStamp"]) -> None: ...

global___DataSourceFileSetStat = DataSourceFileSetStat

@typing.final
class CpRmMvLsReply(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class DataSourceInfo(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        DESC_FIELD_NUMBER: builtins.int
        STAT_FIELD_NUMBER: builtins.int
        @property
        def desc(self) -> common_pb2.FileDescriptor: ...
        @property
        def stat(self) -> global___DataSourceFileSetStat: ...
        def __init__(
            self,
            *,
            desc: common_pb2.FileDescriptor | None = ...,
            stat: global___DataSourceFileSetStat | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["desc", b"desc", "stat", b"stat"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["desc", b"desc", "stat", b"stat"]) -> None: ...

    NUMFILES_FIELD_NUMBER: builtins.int
    NUMDSOURCE_FIELD_NUMBER: builtins.int
    NUMBYTES_FIELD_NUMBER: builtins.int
    DESTPATH_FIELD_NUMBER: builtins.int
    DSOURCE_FIELD_NUMBER: builtins.int
    MSG_FIELD_NUMBER: builtins.int
    numFiles: builtins.int
    numDsource: builtins.int
    numBytes: builtins.int
    destPath: builtins.str
    msg: builtins.str
    @property
    def dsource(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CpRmMvLsReply.DataSourceInfo]: ...
    def __init__(
        self,
        *,
        numFiles: builtins.int | None = ...,
        numDsource: builtins.int | None = ...,
        numBytes: builtins.int | None = ...,
        destPath: builtins.str | None = ...,
        dsource: collections.abc.Iterable[global___CpRmMvLsReply.DataSourceInfo] | None = ...,
        msg: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["destPath", b"destPath", "msg", b"msg", "numBytes", b"numBytes", "numDsource", b"numDsource", "numFiles", b"numFiles"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["destPath", b"destPath", "dsource", b"dsource", "msg", b"msg", "numBytes", b"numBytes", "numDsource", b"numDsource", "numFiles", b"numFiles"]) -> None: ...

global___CpRmMvLsReply = CpRmMvLsReply

@typing.final
class BackgroundProcessStatus(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UID_FIELD_NUMBER: builtins.int
    FRACCOMPLETE_FIELD_NUMBER: builtins.int
    ELAPSEDTIMESEC_FIELD_NUMBER: builtins.int
    ESTIMATEDCOMPLETIONWALLTIME_FIELD_NUMBER: builtins.int
    uID: builtins.str
    fracComplete: builtins.float
    elapsedTimeSec: builtins.float
    estimatedCompletionWallTime: builtins.str
    def __init__(
        self,
        *,
        uID: builtins.str | None = ...,
        fracComplete: builtins.float | None = ...,
        elapsedTimeSec: builtins.float | None = ...,
        estimatedCompletionWallTime: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["elapsedTimeSec", b"elapsedTimeSec", "estimatedCompletionWallTime", b"estimatedCompletionWallTime", "fracComplete", b"fracComplete", "uID", b"uID"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["elapsedTimeSec", b"elapsedTimeSec", "estimatedCompletionWallTime", b"estimatedCompletionWallTime", "fracComplete", b"fracComplete", "uID", b"uID"]) -> None: ...

global___BackgroundProcessStatus = BackgroundProcessStatus

@typing.final
class MoveDataSourceFileRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SRC_FIELD_NUMBER: builtins.int
    DEST_FIELD_NUMBER: builtins.int
    ISFORCE_FIELD_NUMBER: builtins.int
    ISVALIDATE_FIELD_NUMBER: builtins.int
    isForce: builtins.bool
    isValidate: builtins.bool
    @property
    def src(self) -> common_pb2.FileDescriptor: ...
    @property
    def dest(self) -> common_pb2.FileDescriptor: ...
    def __init__(
        self,
        *,
        src: common_pb2.FileDescriptor | None = ...,
        dest: common_pb2.FileDescriptor | None = ...,
        isForce: builtins.bool | None = ...,
        isValidate: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["dest", b"dest", "isForce", b"isForce", "isValidate", b"isValidate", "src", b"src"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dest", b"dest", "isForce", b"isForce", "isValidate", b"isValidate", "src", b"src"]) -> None: ...

global___MoveDataSourceFileRequest = MoveDataSourceFileRequest

@typing.final
class ExportDataSourceFileRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SRC_FIELD_NUMBER: builtins.int
    DEST_FIELD_NUMBER: builtins.int
    ISFORCE_FIELD_NUMBER: builtins.int
    isForce: builtins.bool
    @property
    def src(self) -> common_pb2.FileDescriptor: ...
    @property
    def dest(self) -> common_pb2.FileDescriptor: ...
    def __init__(
        self,
        *,
        src: common_pb2.FileDescriptor | None = ...,
        dest: common_pb2.FileDescriptor | None = ...,
        isForce: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["dest", b"dest", "isForce", b"isForce", "src", b"src"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dest", b"dest", "isForce", b"isForce", "src", b"src"]) -> None: ...

global___ExportDataSourceFileRequest = ExportDataSourceFileRequest

@typing.final
class DataSourceParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SAMPLEFREQ_FIELD_NUMBER: builtins.int
    TIMERANGE_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    DATASOURCETYPE_FIELD_NUMBER: builtins.int
    DATASOURCEID_FIELD_NUMBER: builtins.int
    sampleFreq: builtins.float
    duration: builtins.float
    type: common_pb2.SignalGroupType.ValueType
    datasourceType: global___RadiensDatasourceType.ValueType
    """used by sapiensserver client"""
    datasourceID: builtins.str
    """used by sapiensserver client"""
    @property
    def timeRange(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        sampleFreq: builtins.float | None = ...,
        timeRange: collections.abc.Iterable[builtins.float] | None = ...,
        duration: builtins.float | None = ...,
        type: common_pb2.SignalGroupType.ValueType | None = ...,
        datasourceType: global___RadiensDatasourceType.ValueType | None = ...,
        datasourceID: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["datasourceID", b"datasourceID", "datasourceType", b"datasourceType", "duration", b"duration", "sampleFreq", b"sampleFreq", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["datasourceID", b"datasourceID", "datasourceType", b"datasourceType", "duration", b"duration", "sampleFreq", b"sampleFreq", "timeRange", b"timeRange", "type", b"type"]) -> None: ...

global___DataSourceParams = DataSourceParams

@typing.final
class ImpedanceRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PORT_FIELD_NUMBER: builtins.int
    port: common_pb2.Port.ValueType
    def __init__(
        self,
        *,
        port: common_pb2.Port.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["port", b"port"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["port", b"port"]) -> None: ...

global___ImpedanceRequest = ImpedanceRequest

@typing.final
class Impedance(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class ImpedanceValue(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NTVCHANIDX_FIELD_NUMBER: builtins.int
        ZMAG_FIELD_NUMBER: builtins.int
        ZPHASE_FIELD_NUMBER: builtins.int
        ntvChanIdx: builtins.int
        zMag: builtins.float
        zPhase: builtins.float
        def __init__(
            self,
            *,
            ntvChanIdx: builtins.int | None = ...,
            zMag: builtins.float | None = ...,
            zPhase: builtins.float | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["ntvChanIdx", b"ntvChanIdx", "zMag", b"zMag", "zPhase", b"zPhase"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["ntvChanIdx", b"ntvChanIdx", "zMag", b"zMag", "zPhase", b"zPhase"]) -> None: ...

    IMPEDANCES_FIELD_NUMBER: builtins.int
    @property
    def impedances(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Impedance.ImpedanceValue]: ...
    def __init__(
        self,
        *,
        impedances: collections.abc.Iterable[global___Impedance.ImpedanceValue] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["impedances", b"impedances"]) -> None: ...

global___Impedance = Impedance



"""Test set/get recording config service functionality."""

from unittest.mock import Mock, patch

from radiens_core._generated import allegoserver_pb2, common_pb2
from radiens_core._services._allego_core import (
    get_recording_config_proto,
    set_recording_config_proto,
)
from radiens_core._transport._channel_pool import ChannelPool


class TestSetRecordingConfigProto:
    """Test set_recording_config_proto functionality."""

    def test_calls_stub_with_config_recording_request(self) -> None:
        """Test that set_recording_config_proto calls stub.SetConfigRecording with correct request."""
        # Arrange
        pool = Mock(spec=ChannelPool)
        channel = Mock()
        stub = Mock()
        response = common_pb2.StandardReply()

        pool.get_channel.return_value = channel
        stub.SetConfigRecording.return_value = response

        request = allegoserver_pb2.ConfigRecording(
            baseFileName="test_recording",
            baseFilePath="/path/to/data",
            dataSourceIdx=0,
            timeStamp=True,
            recordingType=allegoserver_pb2.continuous,
        )

        # Mock the stub creation
        with patch(
            "radiens_core._generated.allegoserver_pb2_grpc.AllegoCoreStub",
            return_value=stub,
        ):
            # Act
            result = set_recording_config_proto(pool, "localhost:8080", request)

            # Assert
            pool.get_channel.assert_called_once_with("localhost:8080")
            stub.SetConfigRecording.assert_called_once_with(request)
            assert result is response


class TestGetRecordingConfigProto:
    """Test get_recording_config_proto functionality."""

    def test_calls_stub_with_standard_request(self) -> None:
        """Test that get_recording_config_proto calls stub.GetConfigRecording with StandardRequest."""
        # Arrange
        pool = Mock(spec=ChannelPool)
        channel = Mock()
        stub = Mock()
        response = allegoserver_pb2.ConfigRecording()

        pool.get_channel.return_value = channel
        stub.GetConfigRecording.return_value = response

        # Mock the stub creation
        with patch(
            "radiens_core._generated.allegoserver_pb2_grpc.AllegoCoreStub",
            return_value=stub,
        ):
            # Act
            result = get_recording_config_proto(pool, "localhost:8080")

            # Assert
            pool.get_channel.assert_called_once_with("localhost:8080")
            stub.GetConfigRecording.assert_called_once_with(common_pb2.StandardRequest())
            assert result is response



"""Domain models — pure Pydantic v2, zero proto dependency.

All enums and models are importable from their domain-grouped submodules:

    from radiens_core.models.stim import StimParams, StimShape
    from radiens_core.models.signals import SignalType, SigSelect
    from radiens_core.models.status import RecordMode, StreamMode
"""

from radiens_core.models.channels import (
    AreaUnits,
    ChannelInfo,
    ChannelMetadata,
    KeyIdxs,
    SignalUnits,
    SitePosition,
)
from radiens_core.models.common import ClientType, ServiceEndpoint
from radiens_core.models.core_config import CableDelay, CoreConfig, SetCoreConfigRequest
from radiens_core.models.dac import AnalogOutChannel, AnalogOutRegister, DACHighPass
from radiens_core.models.dataset import (
    BandParams,
    BulkSinkParams,
    BulkSourceParams,
    CARParams,
    DatasetMetadata,
    DownsampleParams,
    HighLowPassParams,
    NotchParams,
    PairedRefParams,
    RadiensFileType,
    SinkParams,
    SliceChannelsParams,
    SliceTimeParams,
    SourceParams,
    TransformEdge,
    TransformNode,
    TransformNodeType,
    TrsMode,
    VirtualRefParams,
)
from radiens_core.models.dio import DigitalOutChannelRegister, DigitalOutRegister, DIOMode
from radiens_core.models.dsp import DSPGroup, DSPParams, DSPType, FilterStage
from radiens_core.models.kpi import KpiMetric, KpiMetricId, KpiMetricsResult, KpiMode, KpiStatus
from radiens_core.models.ports import GPIOChannelCount, Port, PortChannelCount
from radiens_core.models.signals import KeyIndex, SignalArrays, SignalType, SigSelect
from radiens_core.models.spikes import ChannelSpikeData, NeuronInfo, NeuronSpikeData, NeuronsResult, SpikesSpec
from radiens_core.models.status import (
    BackboneMode,
    BackboneStatus,
    DataRecordingType,
    RecordingConfig,
    RecordingStatus,
    RecordMode,
    RecordStateRequest,
    RestartRequest,
    SinapsStatusRegisters,
    StreamingStatus,
    StreamMode,
    StreamStateRequest,
)
from radiens_core.models.stim import (
    StimKeypressIndex,
    StimParams,
    StimPolarity,
    StimPulseMode,
    StimShape,
    StimStep,
    StimStepRequest,
    StimStepState,
    StimTriggerEdgeOrLevel,
    StimTriggerHighOrLow,
)
from radiens_core.models.time_range import TimeRange
from radiens_core.models.triggers import (
    ManualStimTriggerRequest,
    ManualStimTriggerToggleRequest,
    SetTriggerStateRequest,
    TriggerChannel,
    TriggerState,
)

__all__ = [
    "AnalogOutChannel",
    "AnalogOutRegister",
    "AreaUnits",
    "BackboneMode",
    "BackboneStatus",
    "BandParams",
    "BulkSinkParams",
    "BulkSourceParams",
    "CARParams",
    "CableDelay",
    "ChannelInfo",
    "ChannelMetadata",
    "ChannelSpikeData",
    "ClientType",
    "CoreConfig",
    "DACHighPass",
    "DIOMode",
    "DSPGroup",
    "DSPParams",
    "DSPType",
    "DataRecordingType",
    "DatasetMetadata",
    "DigitalOutChannelRegister",
    "DigitalOutRegister",
    "DownsampleParams",
    "FilterStage",
    "GPIOChannelCount",
    "HighLowPassParams",
    "KeyIdxs",
    "KeyIndex",
    "KpiMetric",
    "KpiMetricId",
    "KpiMetricsResult",
    "KpiMode",
    "KpiStatus",
    "ManualStimTriggerRequest",
    "ManualStimTriggerToggleRequest",
    "NeuronInfo",
    "NeuronSpikeData",
    "NeuronsResult",
    "NotchParams",
    "PairedRefParams",
    "Port",
    "PortChannelCount",
    "RadiensFileType",
    "RecordMode",
    "RecordStateRequest",
    "RecordingConfig",
    "RecordingStatus",
    "RestartRequest",
    "ServiceEndpoint",
    "SetCoreConfigRequest",
    "SetTriggerStateRequest",
    "SigSelect",
    "SignalArrays",
    "SignalType",
    "SignalUnits",
    "SinapsStatusRegisters",
    "SinkParams",
    "SitePosition",
    "SliceChannelsParams",
    "SliceTimeParams",
    "SourceParams",
    "SpikesSpec",
    "StimKeypressIndex",
    "StimParams",
    "StimPolarity",
    "StimPulseMode",
    "StimShape",
    "StimStep",
    "StimStepRequest",
    "StimStepState",
    "StimTriggerEdgeOrLevel",
    "StimTriggerHighOrLow",
    "StreamMode",
    "StreamStateRequest",
    "StreamingStatus",
    "TimeRange",
    "TransformEdge",
    "TransformNode",
    "TransformNodeType",
    "TriggerChannel",
    "TriggerState",
    "TrsMode",
    "VirtualRefParams",
]



"""Restart converters — domain models <-> proto messages."""

from __future__ import annotations

from typing import TYPE_CHECKING

from radiens_core._converters._common_enums import (
    backbone_mode_to_proto,
)
from radiens_core._generated import allegoserver_pb2

if TYPE_CHECKING:
    from radiens_core.models.status import BackboneMode, RestartRequest


def restart_request_to_proto(
    request: RestartRequest,
) -> allegoserver_pb2.RestartRequest:
    """Convert domain RestartRequest to proto RestartRequest."""
    return allegoserver_pb2.RestartRequest(
        mode=backbone_mode_to_proto(request.mode),
    )


def build_restart_proto(mode: BackboneMode) -> allegoserver_pb2.RestartRequest:
    """Build a RestartRequest proto directly from a BackboneMode."""
    return allegoserver_pb2.RestartRequest(
        mode=backbone_mode_to_proto(mode),
    )



"""Tests for gRPC error mapping."""

import grpc
import pytest

from radiens_core._transport._errors import handle_grpc_error
from radiens_core.exceptions import DataProcessingError, ServerCommunicationError
from radiens_core.models.common import ClientType


class _FakeRpcError(grpc.RpcError):
    def __init__(self, status_code: grpc.StatusCode, details: str) -> None:
        self._status_code = status_code
        self._details = details

    def code(self) -> grpc.StatusCode:
        return self._status_code

    def details(self) -> str:
        return self._details


def _make_rpc_error(status_code: grpc.StatusCode, details: str = "test") -> grpc.RpcError:
    return _FakeRpcError(status_code, details)


def test_unavailable_maps_to_server_error() -> None:
    ex = _make_rpc_error(grpc.StatusCode.UNAVAILABLE)
    with pytest.raises(ServerCommunicationError, match="Cannot connect"):
        handle_grpc_error(ex, ClientType.VIDERE)


def test_unauthenticated_maps_to_server_error() -> None:
    ex = _make_rpc_error(grpc.StatusCode.UNAUTHENTICATED)
    with pytest.raises(ServerCommunicationError, match="Authentication failed"):
        handle_grpc_error(ex, ClientType.ALLEGO)


def test_resource_exhausted_maps_to_data_error() -> None:
    ex = _make_rpc_error(grpc.StatusCode.RESOURCE_EXHAUSTED)
    with pytest.raises(DataProcessingError, match="too large"):
        handle_grpc_error(ex, ClientType.VIDERE)


def test_unknown_maps_to_server_error() -> None:
    ex = _make_rpc_error(grpc.StatusCode.UNKNOWN)
    with pytest.raises(ServerCommunicationError, match="Unknown"):
        handle_grpc_error(ex, ClientType.CURATE)


def test_other_status_maps_to_server_error() -> None:
    ex = _make_rpc_error(grpc.StatusCode.INTERNAL)
    with pytest.raises(ServerCommunicationError, match="gRPC error"):
        handle_grpc_error(ex, ClientType.VIDERE)



"""Channel metadata, site position models, and channel-related enums."""

from __future__ import annotations

from enum import IntEnum
from typing import assert_never

import numpy as np
import numpy.typing as npt
from pydantic import BaseModel, ConfigDict

from radiens_core.exceptions import DatasetError
from radiens_core.models.signals import SignalType


class SignalUnits(IntEnum):
    """Signal measurement units."""

    UNKNOWN = 0
    MICROVOLTS = 1
    VOLTS = 2
    BINARY = 3


class AreaUnits(IntEnum):
    """Area measurement units for electrode sites."""

    SQUARE_MICROMETERS = 0  # µm² (base unit)
    SQUARE_MILLIMETERS = 1  # mm²
    SQUARE_CENTIMETERS = 2  # cm²
    SQUARE_NANOMETERS = 3  # nm²
    SQUARE_METERS = 4  # m²


class SitePosition(BaseModel):
    """Electrode site position and geometry in probe and tissue coordinates.

    Attributes:
        probe_x: X-coordinate in probe coordinate system.
        probe_y: Y-coordinate in probe coordinate system.
        probe_z: Z-coordinate in probe coordinate system.
        tissue_x: X-coordinate in tissue coordinate system.
        tissue_y: Y-coordinate in tissue coordinate system.
        tissue_z: Z-coordinate in tissue coordinate system.
        site_shape: Geometric shape of the electrode site.
        site_area_um2: Surface area of the electrode site in square micrometers.
        site_lim_x_min: Minimum X-limit of the site bounding box.
        site_lim_x_max: Maximum X-limit of the site bounding box.
        site_lim_y_min: Minimum Y-limit of the site bounding box.
        site_lim_y_max: Maximum Y-limit of the site bounding box.
        site_lim_z_min: Minimum Z-limit of the site bounding box.
        site_lim_z_max: Maximum Z-limit of the site bounding box.
        scs_to_tcs_x: X-offset for probe to tissue coordinate transformation.
        scs_to_tcs_y: Y-offset for probe to tissue coordinate transformation.
        scs_to_tcs_z: Z-offset for probe to tissue coordinate transformation.
    """

    model_config = ConfigDict(frozen=True)
    # Basic coordinates
    probe_x: float
    probe_y: float
    probe_z: float
    tissue_x: float
    tissue_y: float
    tissue_z: float

    # Geometry metadata
    site_shape: str
    site_area_um2: float

    # Bounding box limits (probe coordinates)
    site_lim_x_min: float
    site_lim_x_max: float
    site_lim_y_min: float
    site_lim_y_max: float
    site_lim_z_min: float
    site_lim_z_max: float

    # Coordinate system transformation offsets
    scs_to_tcs_x: float
    scs_to_tcs_y: float
    scs_to_tcs_z: float

    @property
    def bounding_box_size(self) -> tuple[float, float, float]:
        """Site bounding box size (width, height, depth) in micrometers."""
        return (
            self.site_lim_x_max - self.site_lim_x_min,
            self.site_lim_y_max - self.site_lim_y_min,
            self.site_lim_z_max - self.site_lim_z_min,
        )

    @property
    def bounding_box_center(self) -> tuple[float, float, float]:
        """Site bounding box center coordinates in probe coordinate system."""
        return (
            (self.site_lim_x_min + self.site_lim_x_max) / 2,
            (self.site_lim_y_min + self.site_lim_y_max) / 2,
            (self.site_lim_z_min + self.site_lim_z_max) / 2,
        )

    def get_area(self, unit: AreaUnits) -> float:
        """Get site area in the specified units.

        Args:
            unit: Target area unit for conversion

        Returns:
            Site area converted to the specified unit
        """
        # Base area is in µm²
        base_area_um2 = self.site_area_um2

        if unit == AreaUnits.SQUARE_MICROMETERS:
            return base_area_um2
        elif unit == AreaUnits.SQUARE_MILLIMETERS:
            # 1 mm² = 1,000,000 µm²
            return base_area_um2 / 1_000_000
        elif unit == AreaUnits.SQUARE_CENTIMETERS:
            # 1 cm² = 100,000,000 µm²
            return base_area_um2 / 100_000_000
        elif unit == AreaUnits.SQUARE_NANOMETERS:
            # 1 µm² = 1,000,000 nm²
            return base_area_um2 * 1_000_000
        elif unit == AreaUnits.SQUARE_METERS:
            # 1 m² = 1,000,000,000,000 µm²
            return base_area_um2 / 1_000_000_000_000
        else:
            assert_never(unit)


class KeyIdxs(BaseModel):
    """Channel indices in all three key index spaces.

    Attributes:
        ntv: Native channel indices.
        dset: Dataset channel indices.
        sys: System channel indices.
    """

    model_config = ConfigDict(frozen=True)
    ntv: list[int]
    dset: list[int]
    sys: list[int]


class ChannelInfo(BaseModel):
    """Complete channel information for a specific channel.

    Attributes:
        ntv_idx: Native channel index.
        dset_idx: Dataset channel index.
        sys_idx: System channel index.
        signal_type: Type of the signal. References SignalType.
        site_position: Site position and geometry. Defaults to None.
    """

    model_config = ConfigDict(frozen=True)
    ntv_idx: int
    dset_idx: int
    sys_idx: int
    signal_type: SignalType
    site_position: SitePosition | None = None


class ChannelMetadata(BaseModel):
    """Channel metadata for a dataset.

    Contains channel configuration, probe geometry, signal types,
    and indices for all channels.

    Attributes:
        dataset_uid: Unique identifier for the dataset.
        fs: Sampling frequency in Hertz.
        source_label: Label for the data source.
        neighbor_max_radius_um: Maximum radius for neighbor search in micrometers.
        signal_units: Mapping of signal types to their units. References SignalType and SignalUnits.
        has_discrete: Whether the dataset contains discrete signals.
        has_continuous_amp: Whether the dataset contains continuous amplifier signals.
        sig_map: Mapping of signal types to their indices. References SignalType and KeyIdxs.
        selected_sig_map: Mapping of signal types to their selected indices. References SignalType and KeyIdxs.
        num_channels: Total number of channels per signal type. References SignalType.
        selected_num_channels: Number of selected channels per signal type. References SignalType.
        site_positions: List of site positions for amplifier channels.
    """

    model_config = ConfigDict(frozen=True, arbitrary_types_allowed=True)
    dataset_uid: str
    fs: float
    source_label: str
    neighbor_max_radius_um: float
    signal_units: dict[SignalType, SignalUnits]
    has_discrete: bool
    has_continuous_amp: bool

    # Channel indices per signal type
    sig_map: dict[SignalType, KeyIdxs]
    selected_sig_map: dict[SignalType, KeyIdxs]
    num_channels: dict[SignalType, int]
    selected_num_channels: dict[SignalType, int]

    # Site positions (one per amp channel, native index order)
    site_positions: list[SitePosition]

    def index(self, sig_type: SignalType) -> KeyIdxs:
        """Get channel indices for a signal type."""
        return self.sig_map[sig_type]

    def selected_index(self, sig_type: SignalType) -> KeyIdxs:
        """Get selected channel indices for a signal type."""
        return self.selected_sig_map[sig_type]

    def get_num_channels(self, sig_type: SignalType) -> int:
        """Get total number of channels for a signal type."""
        return self.num_channels[sig_type]

    def all_amp_indices(self) -> npt.NDArray[np.int64]:
        """Get all amplifier channel native indices."""
        return np.array(self.sig_map[SignalType.AMP].ntv, dtype=np.int64)

    def get_channel_info(
        self,
        ntv_idx: int,
        sig_type: SignalType,
    ) -> ChannelInfo:
        """Get complete channel information from native index.

        Args:
            ntv_idx: Native channel index
            sig_type: Signal type to look up

        Returns:
            Complete channel information including indices and site position

        Raises:
            DatasetError: If ntv_idx not found for the signal type
        """
        key_idxs = self.sig_map[sig_type]

        try:
            position = key_idxs.ntv.index(ntv_idx)
        except ValueError:
            msg = f"NTV index {ntv_idx} not found for signal type {sig_type}"
            raise DatasetError(msg) from None

        site_position = None
        if sig_type == SignalType.AMP and position < len(self.site_positions):
            site_position = self.site_positions[position]

        return ChannelInfo(
            ntv_idx=ntv_idx,
            dset_idx=key_idxs.dset[position],
            sys_idx=key_idxs.sys[position],
            signal_type=sig_type,
            site_position=site_position,
        )

    def ntv_from_dset(self, dset_idx: int, sig_type: SignalType) -> int:
        """Convert dataset index to native index.

        Args:
            dset_idx: Dataset index
            sig_type: Signal type

        Returns:
            Native index for the channel

        Raises:
            DatasetError: If dset_idx not found for the signal type
        """
        key_idxs = self.sig_map[sig_type]
        try:
            position = key_idxs.dset.index(dset_idx)
            return key_idxs.ntv[position]
        except ValueError:
            msg = f"Dataset index {dset_idx} not found for signal type {sig_type}"
            raise DatasetError(msg) from None

    def ntv_from_sys(self, sys_idx: int, sig_type: SignalType) -> int:
        """Convert system index to native index.

        Args:
            sys_idx: System index
            sig_type: Signal type

        Returns:
            Native index for the channel

        Raises:
            DatasetError: If sys_idx not found for the signal type
        """
        key_idxs = self.sig_map[sig_type]
        try:
            position = key_idxs.sys.index(sys_idx)
            return key_idxs.ntv[position]
        except ValueError:
            msg = f"System index {sys_idx} not found for signal type {sig_type}"
            raise DatasetError(msg) from None

    @property
    def amp_site_count(self) -> int:
        """Number of amplifier channel sites."""
        return len(self.site_positions)

    @property
    def total_selected_channels(self) -> int:
        """Total number of selected channels across all signal types."""
        return sum(self.selected_num_channels.values())



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from . import biointerface_pb2
import builtins
import collections.abc
from . import common_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _PyBridgeFileIOmode:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _PyBridgeFileIOmodeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_PyBridgeFileIOmode.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    READ: _PyBridgeFileIOmode.ValueType  # 0
    WRITE: _PyBridgeFileIOmode.ValueType  # 1

class PyBridgeFileIOmode(_PyBridgeFileIOmode, metaclass=_PyBridgeFileIOmodeEnumTypeWrapper): ...

READ: PyBridgeFileIOmode.ValueType  # 0
WRITE: PyBridgeFileIOmode.ValueType  # 1
global___PyBridgeFileIOmode = PyBridgeFileIOmode

class _SpikesExportFileMode:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _SpikesExportFileModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_SpikesExportFileMode.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SPIKES_FILE_CREATE: _SpikesExportFileMode.ValueType  # 0
    SPIKES_FILE_APPEND: _SpikesExportFileMode.ValueType  # 1
    SPIKES_FILE_POST_HOC: _SpikesExportFileMode.ValueType  # 2

class SpikesExportFileMode(_SpikesExportFileMode, metaclass=_SpikesExportFileModeEnumTypeWrapper): ...

SPIKES_FILE_CREATE: SpikesExportFileMode.ValueType  # 0
SPIKES_FILE_APPEND: SpikesExportFileMode.ValueType  # 1
SPIKES_FILE_POST_HOC: SpikesExportFileMode.ValueType  # 2
global___SpikesExportFileMode = SpikesExportFileMode

class _WaveformType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _WaveformTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_WaveformType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SPIKE: _WaveformType.ValueType  # 0
    TEMPLATE: _WaveformType.ValueType  # 1

class WaveformType(_WaveformType, metaclass=_WaveformTypeEnumTypeWrapper): ...

SPIKE: WaveformType.ValueType  # 0
TEMPLATE: WaveformType.ValueType  # 1
global___WaveformType = WaveformType

@typing.final
class Request(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    @property
    def data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        data: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["data", b"data"]) -> None: ...

global___Request = Request

@typing.final
class Reply(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    @property
    def data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        data: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["data", b"data"]) -> None: ...

global___Reply = Reply

@typing.final
class PyBridgeFileStandardRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FILEUID_FIELD_NUMBER: builtins.int
    fileUID: builtins.str
    def __init__(
        self,
        *,
        fileUID: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["fileUID", b"fileUID"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["fileUID", b"fileUID"]) -> None: ...

global___PyBridgeFileStandardRequest = PyBridgeFileStandardRequest

@typing.final
class PyBridgeFileOpenRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FILEUID_FIELD_NUMBER: builtins.int
    FILETYPE_FIELD_NUMBER: builtins.int
    IOMODE_FIELD_NUMBER: builtins.int
    FILENAME_FIELD_NUMBER: builtins.int
    fileUID: builtins.str
    fileType: common_pb2.RadixFileTypes.ValueType
    ioMode: global___PyBridgeFileIOmode.ValueType
    fileName: builtins.str
    def __init__(
        self,
        *,
        fileUID: builtins.str | None = ...,
        fileType: common_pb2.RadixFileTypes.ValueType | None = ...,
        ioMode: global___PyBridgeFileIOmode.ValueType | None = ...,
        fileName: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["fileName", b"fileName", "fileType", b"fileType", "fileUID", b"fileUID", "ioMode", b"ioMode"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["fileName", b"fileName", "fileType", b"fileType", "fileUID", b"fileUID", "ioMode", b"ioMode"]) -> None: ...

global___PyBridgeFileOpenRequest = PyBridgeFileOpenRequest

@typing.final
class PyBridgeIirFilterDesignRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FILTTYPE_FIELD_NUMBER: builtins.int
    FREQSPEC_FIELD_NUMBER: builtins.int
    ORDER_FIELD_NUMBER: builtins.int
    SAMPFREQ_FIELD_NUMBER: builtins.int
    SECONDORDERSECTION_FIELD_NUMBER: builtins.int
    filtType: common_pb2.DSPType.ValueType
    order: builtins.int
    sampFreq: builtins.float
    secondOrderSection: builtins.bool
    """if true, design will be in SOS form"""
    @property
    def freqSpec(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        filtType: common_pb2.DSPType.ValueType | None = ...,
        freqSpec: collections.abc.Iterable[builtins.float] | None = ...,
        order: builtins.int | None = ...,
        sampFreq: builtins.float | None = ...,
        secondOrderSection: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["filtType", b"filtType", "order", b"order", "sampFreq", b"sampFreq", "secondOrderSection", b"secondOrderSection"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["filtType", b"filtType", "freqSpec", b"freqSpec", "order", b"order", "sampFreq", b"sampFreq", "secondOrderSection", b"secondOrderSection"]) -> None: ...

global___PyBridgeIirFilterDesignRequest = PyBridgeIirFilterDesignRequest

@typing.final
class PyBridgeIirFilterDesignReply(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HA_FIELD_NUMBER: builtins.int
    HB_FIELD_NUMBER: builtins.int
    @property
    def ha(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def hb(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        ha: collections.abc.Iterable[builtins.float] | None = ...,
        hb: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["ha", b"ha", "hb", b"hb"]) -> None: ...

global___PyBridgeIirFilterDesignReply = PyBridgeIirFilterDesignReply

@typing.final
class PyBridgeGetSpecReply(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FILEUID_FIELD_NUMBER: builtins.int
    DATAFILENAME_FIELD_NUMBER: builtins.int
    FILETYPE_FIELD_NUMBER: builtins.int
    SHAPEPRI_FIELD_NUMBER: builtins.int
    SHAPEAUX_FIELD_NUMBER: builtins.int
    SHAPEDIN_FIELD_NUMBER: builtins.int
    SHAPEDOUT_FIELD_NUMBER: builtins.int
    SAMPFREQ_FIELD_NUMBER: builtins.int
    TIMESPEC_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    fileUID: builtins.str
    dataFileName: builtins.str
    fileType: common_pb2.RadixFileTypes.ValueType
    sampFreq: builtins.float
    @property
    def shapePri(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def shapeAux(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def shapeDin(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def shapeDout(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def timeSpec(self) -> global___PyBridgeTimeSpec: ...
    @property
    def metaData(self) -> global___PyBridgeFileMetaData: ...
    def __init__(
        self,
        *,
        fileUID: builtins.str | None = ...,
        dataFileName: builtins.str | None = ...,
        fileType: common_pb2.RadixFileTypes.ValueType | None = ...,
        shapePri: collections.abc.Iterable[builtins.int] | None = ...,
        shapeAux: collections.abc.Iterable[builtins.int] | None = ...,
        shapeDin: collections.abc.Iterable[builtins.int] | None = ...,
        shapeDout: collections.abc.Iterable[builtins.int] | None = ...,
        sampFreq: builtins.float | None = ...,
        timeSpec: global___PyBridgeTimeSpec | None = ...,
        metaData: global___PyBridgeFileMetaData | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["dataFileName", b"dataFileName", "fileType", b"fileType", "fileUID", b"fileUID", "metaData", b"metaData", "sampFreq", b"sampFreq", "timeSpec", b"timeSpec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dataFileName", b"dataFileName", "fileType", b"fileType", "fileUID", b"fileUID", "metaData", b"metaData", "sampFreq", b"sampFreq", "shapeAux", b"shapeAux", "shapeDin", b"shapeDin", "shapeDout", b"shapeDout", "shapePri", b"shapePri", "timeSpec", b"timeSpec"]) -> None: ...

global___PyBridgeGetSpecReply = PyBridgeGetSpecReply

@typing.final
class PyBridgeTimeSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class PyBridgeTimeRangeT(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TIMERANGE_FIELD_NUMBER: builtins.int
        @property
        def timeRange(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
        def __init__(
            self,
            *,
            timeRange: collections.abc.Iterable[builtins.float] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["timeRange", b"timeRange"]) -> None: ...

    @typing.final
    class PyBridgeTimestampRangeT(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TIMESTAMPRANGE_FIELD_NUMBER: builtins.int
        @property
        def timestampRange(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        def __init__(
            self,
            *,
            timestampRange: collections.abc.Iterable[builtins.int] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["timestampRange", b"timestampRange"]) -> None: ...

    @typing.final
    class PyBridgeDataBlockRangeT(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NUMDATABLOCKS_FIELD_NUMBER: builtins.int
        NUMSAMPLESPERDATABLOCK_FIELD_NUMBER: builtins.int
        FIRSTTIMESTAMP_FIELD_NUMBER: builtins.int
        HEADERBYTES_FIELD_NUMBER: builtins.int
        NUMBYTESDATABLOCK_FIELD_NUMBER: builtins.int
        numDataBlocks: builtins.int
        numSamplesPerDataBlock: builtins.int
        firstTimestamp: builtins.int
        headerBytes: builtins.int
        numBytesDataBlock: builtins.int
        def __init__(
            self,
            *,
            numDataBlocks: builtins.int | None = ...,
            numSamplesPerDataBlock: builtins.int | None = ...,
            firstTimestamp: builtins.int | None = ...,
            headerBytes: builtins.int | None = ...,
            numBytesDataBlock: builtins.int | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["firstTimestamp", b"firstTimestamp", "headerBytes", b"headerBytes", "numBytesDataBlock", b"numBytesDataBlock", "numDataBlocks", b"numDataBlocks", "numSamplesPerDataBlock", b"numSamplesPerDataBlock"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["firstTimestamp", b"firstTimestamp", "headerBytes", b"headerBytes", "numBytesDataBlock", b"numBytesDataBlock", "numDataBlocks", b"numDataBlocks", "numSamplesPerDataBlock", b"numSamplesPerDataBlock"]) -> None: ...

    TIMERANGE_FIELD_NUMBER: builtins.int
    TIMESTAMPRANGE_FIELD_NUMBER: builtins.int
    DATABLOCK_FIELD_NUMBER: builtins.int
    @property
    def timeRange(self) -> global___PyBridgeTimeSpec.PyBridgeTimeRangeT: ...
    @property
    def timestampRange(self) -> global___PyBridgeTimeSpec.PyBridgeTimestampRangeT: ...
    @property
    def dataBlock(self) -> global___PyBridgeTimeSpec.PyBridgeDataBlockRangeT: ...
    def __init__(
        self,
        *,
        timeRange: global___PyBridgeTimeSpec.PyBridgeTimeRangeT | None = ...,
        timestampRange: global___PyBridgeTimeSpec.PyBridgeTimestampRangeT | None = ...,
        dataBlock: global___PyBridgeTimeSpec.PyBridgeDataBlockRangeT | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["dataBlock", b"dataBlock", "timeRange", b"timeRange", "timestampRange", b"timestampRange"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dataBlock", b"dataBlock", "timeRange", b"timeRange", "timestampRange", b"timestampRange"]) -> None: ...

global___PyBridgeTimeSpec = PyBridgeTimeSpec

@typing.final
class PyBridgePortMap(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class PyBridgeSitePos(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        POSXYZ_FIELD_NUMBER: builtins.int
        @property
        def posXYZ(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
        def __init__(
            self,
            *,
            posXYZ: collections.abc.Iterable[builtins.float] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["posXYZ", b"posXYZ"]) -> None: ...

    @typing.final
    class PortIdxToDfSigMapEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        value: builtins.int
        def __init__(
            self,
            *,
            key: builtins.int | None = ...,
            value: builtins.int | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class DfChanIdxToDfSitePosEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        @property
        def value(self) -> global___PyBridgePortMap.PyBridgeSitePos: ...
        def __init__(
            self,
            *,
            key: builtins.int | None = ...,
            value: global___PyBridgePortMap.PyBridgeSitePos | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    PROBEID_FIELD_NUMBER: builtins.int
    HEADSTAGEID_FIELD_NUMBER: builtins.int
    DFDEVICELABEL_FIELD_NUMBER: builtins.int
    PORTIDXTODFSIGMAP_FIELD_NUMBER: builtins.int
    DFCHANIDXTODFSITEPOS_FIELD_NUMBER: builtins.int
    probeID: builtins.str
    """sapiens probe ID"""
    headstageID: builtins.str
    """sapiens headstage ID"""
    DfDeviceLabel: builtins.str
    """datafile device label"""
    @property
    def portIdxToDfSigMap(self) -> google.protobuf.internal.containers.ScalarMap[builtins.int, builtins.int]:
        """key=port indx"""

    @property
    def dfChanIdxToDfSitePos(self) -> google.protobuf.internal.containers.MessageMap[builtins.int, global___PyBridgePortMap.PyBridgeSitePos]:
        """key=datafile channel idx"""

    def __init__(
        self,
        *,
        probeID: builtins.str | None = ...,
        headstageID: builtins.str | None = ...,
        DfDeviceLabel: builtins.str | None = ...,
        portIdxToDfSigMap: collections.abc.Mapping[builtins.int, builtins.int] | None = ...,
        dfChanIdxToDfSitePos: collections.abc.Mapping[builtins.int, global___PyBridgePortMap.PyBridgeSitePos] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["DfDeviceLabel", b"DfDeviceLabel", "headstageID", b"headstageID", "probeID", b"probeID"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["DfDeviceLabel", b"DfDeviceLabel", "dfChanIdxToDfSitePos", b"dfChanIdxToDfSitePos", "headstageID", b"headstageID", "portIdxToDfSigMap", b"portIdxToDfSigMap", "probeID", b"probeID"]) -> None: ...

global___PyBridgePortMap = PyBridgePortMap

@typing.final
class PyBridgeFileMetaData(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class PyBridgePorts(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        @typing.final
        class PortMapEntry(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            KEY_FIELD_NUMBER: builtins.int
            VALUE_FIELD_NUMBER: builtins.int
            key: builtins.int
            @property
            def value(self) -> global___PyBridgePortMap: ...
            def __init__(
                self,
                *,
                key: builtins.int | None = ...,
                value: global___PyBridgePortMap | None = ...,
            ) -> None: ...
            def HasField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> builtins.bool: ...
            def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

        PORTMAP_FIELD_NUMBER: builtins.int
        @property
        def portMap(self) -> google.protobuf.internal.containers.MessageMap[builtins.int, global___PyBridgePortMap]:
            """key=port enum"""

        def __init__(
            self,
            *,
            portMap: collections.abc.Mapping[builtins.int, global___PyBridgePortMap] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["portMap", b"portMap"]) -> None: ...

    @typing.final
    class MetadataRhd(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        VERSION_FIELD_NUMBER: builtins.int
        NOTES_FIELD_NUMBER: builtins.int
        PORTSIGS_FIELD_NUMBER: builtins.int
        ISFILTERENABLED_FIELD_NUMBER: builtins.int
        CUTOFFFREQ_FIELD_NUMBER: builtins.int
        LOWERBANDWIDTH_FIELD_NUMBER: builtins.int
        UPPERBANDWIDTH_FIELD_NUMBER: builtins.int
        NOTCHFILTERFREQ_FIELD_NUMBER: builtins.int
        version: builtins.str
        notes: builtins.str
        isFilterEnabled: builtins.bool
        cutoffFreq: builtins.float
        lowerBandwidth: builtins.float
        upperBandwidth: builtins.float
        notchFilterFreq: builtins.float
        @property
        def portSigs(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
            """len=8 (max # of ports), # number of signals on each ordered port"""

        def __init__(
            self,
            *,
            version: builtins.str | None = ...,
            notes: builtins.str | None = ...,
            portSigs: collections.abc.Iterable[builtins.int] | None = ...,
            isFilterEnabled: builtins.bool | None = ...,
            cutoffFreq: builtins.float | None = ...,
            lowerBandwidth: builtins.float | None = ...,
            upperBandwidth: builtins.float | None = ...,
            notchFilterFreq: builtins.float | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["cutoffFreq", b"cutoffFreq", "isFilterEnabled", b"isFilterEnabled", "lowerBandwidth", b"lowerBandwidth", "notchFilterFreq", b"notchFilterFreq", "notes", b"notes", "upperBandwidth", b"upperBandwidth", "version", b"version"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["cutoffFreq", b"cutoffFreq", "isFilterEnabled", b"isFilterEnabled", "lowerBandwidth", b"lowerBandwidth", "notchFilterFreq", b"notchFilterFreq", "notes", b"notes", "portSigs", b"portSigs", "upperBandwidth", b"upperBandwidth", "version", b"version"]) -> None: ...

    PORTDESC_FIELD_NUMBER: builtins.int
    RHD_FIELD_NUMBER: builtins.int
    ANNOTATE_FIELD_NUMBER: builtins.int
    STARTTIMEUNIX_FIELD_NUMBER: builtins.int
    startTimeUnix: builtins.int
    @property
    def portDesc(self) -> global___PyBridgeFileMetaData.PyBridgePorts:
        """provided via ancillary file"""

    @property
    def rhd(self) -> global___PyBridgeFileMetaData.MetadataRhd:
        """scraped from rhd file header"""

    @property
    def annotate(self) -> common_pb2.AnnotateBundle: ...
    def __init__(
        self,
        *,
        portDesc: global___PyBridgeFileMetaData.PyBridgePorts | None = ...,
        rhd: global___PyBridgeFileMetaData.MetadataRhd | None = ...,
        annotate: common_pb2.AnnotateBundle | None = ...,
        startTimeUnix: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["annotate", b"annotate", "portDesc", b"portDesc", "rhd", b"rhd", "startTimeUnix", b"startTimeUnix"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["annotate", b"annotate", "portDesc", b"portDesc", "rhd", b"rhd", "startTimeUnix", b"startTimeUnix"]) -> None: ...

global___PyBridgeFileMetaData = PyBridgeFileMetaData

@typing.final
class PyBridgeGetStatusReply(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIMESTAMPRANGE_FIELD_NUMBER: builtins.int
    @property
    def timestampRange(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        timestampRange: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["timestampRange", b"timestampRange"]) -> None: ...

global___PyBridgeGetStatusReply = PyBridgeGetStatusReply

@typing.final
class PyBridgeGetSignalsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FILEID_FIELD_NUMBER: builtins.int
    TIMESTAMPRANGE_FIELD_NUMBER: builtins.int
    FILETYPE_FIELD_NUMBER: builtins.int
    fileID: builtins.str
    fileType: common_pb2.RadixFileTypes.ValueType
    @property
    def timestampRange(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        fileID: builtins.str | None = ...,
        timestampRange: collections.abc.Iterable[builtins.int] | None = ...,
        fileType: common_pb2.RadixFileTypes.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["fileID", b"fileID", "fileType", b"fileType"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["fileID", b"fileID", "fileType", b"fileType", "timestampRange", b"timestampRange"]) -> None: ...

global___PyBridgeGetSignalsRequest = PyBridgeGetSignalsRequest

@typing.final
class PyBridgeGetSignalsReply(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FILEID_FIELD_NUMBER: builtins.int
    TIMESTAMPRANGE_FIELD_NUMBER: builtins.int
    SIGDATAPRI_FIELD_NUMBER: builtins.int
    SIGSHAPEPRI_FIELD_NUMBER: builtins.int
    SIGDATAAUX_FIELD_NUMBER: builtins.int
    SIGSHAPEAUX_FIELD_NUMBER: builtins.int
    SIGDATADIN_FIELD_NUMBER: builtins.int
    SIGSHAPEDIN_FIELD_NUMBER: builtins.int
    SIGDATADOUT_FIELD_NUMBER: builtins.int
    SIGSHAPEDOUT_FIELD_NUMBER: builtins.int
    fileID: builtins.str
    sigDataPri: builtins.bytes
    sigDataAux: builtins.bytes
    sigDataDin: builtins.bytes
    sigDataDout: builtins.bytes
    @property
    def timestampRange(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def sigShapePri(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def sigShapeAux(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def sigShapeDin(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def sigShapeDout(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        fileID: builtins.str | None = ...,
        timestampRange: collections.abc.Iterable[builtins.int] | None = ...,
        sigDataPri: builtins.bytes | None = ...,
        sigShapePri: collections.abc.Iterable[builtins.int] | None = ...,
        sigDataAux: builtins.bytes | None = ...,
        sigShapeAux: collections.abc.Iterable[builtins.int] | None = ...,
        sigDataDin: builtins.bytes | None = ...,
        sigShapeDin: collections.abc.Iterable[builtins.int] | None = ...,
        sigDataDout: builtins.bytes | None = ...,
        sigShapeDout: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["fileID", b"fileID", "sigDataAux", b"sigDataAux", "sigDataDin", b"sigDataDin", "sigDataDout", b"sigDataDout", "sigDataPri", b"sigDataPri"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["fileID", b"fileID", "sigDataAux", b"sigDataAux", "sigDataDin", b"sigDataDin", "sigDataDout", b"sigDataDout", "sigDataPri", b"sigDataPri", "sigShapeAux", b"sigShapeAux", "sigShapeDin", b"sigShapeDin", "sigShapeDout", b"sigShapeDout", "sigShapePri", b"sigShapePri", "timestampRange", b"timestampRange"]) -> None: ...

global___PyBridgeGetSignalsReply = PyBridgeGetSignalsReply

@typing.final
class MearecSignalRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIMESTAMPRANGE_FIELD_NUMBER: builtins.int
    @property
    def timestampRange(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        timestampRange: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["timestampRange", b"timestampRange"]) -> None: ...

global___MearecSignalRequest = MearecSignalRequest

@typing.final
class MearecSignalReply(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    SHAPE_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    @property
    def shape(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        data: builtins.bytes | None = ...,
        shape: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["data", b"data"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["data", b"data", "shape", b"shape"]) -> None: ...

global___MearecSignalReply = MearecSignalReply

@typing.final
class MearecTemplateRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TEMPLATEIDX_FIELD_NUMBER: builtins.int
    templateIdx: builtins.int
    def __init__(
        self,
        *,
        templateIdx: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["templateIdx", b"templateIdx"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["templateIdx", b"templateIdx"]) -> None: ...

global___MearecTemplateRequest = MearecTemplateRequest

@typing.final
class MearecWaveformRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIMERANGESEC_FIELD_NUMBER: builtins.int
    @property
    def timeRangeSec(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        timeRangeSec: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["timeRangeSec", b"timeRangeSec"]) -> None: ...

global___MearecWaveformRequest = MearecWaveformRequest

@typing.final
class SpikesPushToExportReq(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PATHANDFILENAME_FIELD_NUMBER: builtins.int
    FILETYPE_FIELD_NUMBER: builtins.int
    MODE_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    SIGMAP_FIELD_NUMBER: builtins.int
    SPKMTXSET_FIELD_NUMBER: builtins.int
    NEUSPKMTXSET_FIELD_NUMBER: builtins.int
    pathAndFileName: builtins.str
    """output hdf5 file"""
    fileType: common_pb2.RadixFileTypes.ValueType
    """output hdf5 file type"""
    mode: global___SpikesExportFileMode.ValueType
    @property
    def metadata(self) -> global___SpikesExportMetadata: ...
    @property
    def sigMap(self) -> common_pb2.SignalMap: ...
    @property
    def spkMtxSet(self) -> biointerface_pb2.SpikeMatrixSet: ...
    @property
    def neuSpkMtxSet(self) -> biointerface_pb2.NeuronSpikeMatrixSet: ...
    def __init__(
        self,
        *,
        pathAndFileName: builtins.str | None = ...,
        fileType: common_pb2.RadixFileTypes.ValueType | None = ...,
        mode: global___SpikesExportFileMode.ValueType | None = ...,
        metadata: global___SpikesExportMetadata | None = ...,
        sigMap: common_pb2.SignalMap | None = ...,
        spkMtxSet: biointerface_pb2.SpikeMatrixSet | None = ...,
        neuSpkMtxSet: biointerface_pb2.NeuronSpikeMatrixSet | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["fileType", b"fileType", "metadata", b"metadata", "mode", b"mode", "neuSpkMtxSet", b"neuSpkMtxSet", "pathAndFileName", b"pathAndFileName", "sigMap", b"sigMap", "spkMtxSet", b"spkMtxSet"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["fileType", b"fileType", "metadata", b"metadata", "mode", b"mode", "neuSpkMtxSet", b"neuSpkMtxSet", "pathAndFileName", b"pathAndFileName", "sigMap", b"sigMap", "spkMtxSet", b"spkMtxSet"]) -> None: ...

global___SpikesPushToExportReq = SpikesPushToExportReq

@typing.final
class SpikesExportMetadata(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NUMSPIKES_FIELD_NUMBER: builtins.int
    NUMSITES_FIELD_NUMBER: builtins.int
    NUMNEURONS_FIELD_NUMBER: builtins.int
    DATASETUID_FIELD_NUMBER: builtins.int
    SAMPFREQ_FIELD_NUMBER: builtins.int
    TIMERANGE_FIELD_NUMBER: builtins.int
    TIMESTAMPRANGE_FIELD_NUMBER: builtins.int
    numSpikes: builtins.int
    numSites: builtins.int
    numNeurons: builtins.int
    datasetUID: builtins.str
    sampFreq: builtins.float
    @property
    def timeRange(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def timestampRange(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        numSpikes: builtins.int | None = ...,
        numSites: builtins.int | None = ...,
        numNeurons: builtins.int | None = ...,
        datasetUID: builtins.str | None = ...,
        sampFreq: builtins.float | None = ...,
        timeRange: collections.abc.Iterable[builtins.float] | None = ...,
        timestampRange: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["datasetUID", b"datasetUID", "numNeurons", b"numNeurons", "numSites", b"numSites", "numSpikes", b"numSpikes", "sampFreq", b"sampFreq"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["datasetUID", b"datasetUID", "numNeurons", b"numNeurons", "numSites", b"numSites", "numSpikes", b"numSpikes", "sampFreq", b"sampFreq", "timeRange", b"timeRange", "timestampRange", b"timestampRange"]) -> None: ...

global___SpikesExportMetadata = SpikesExportMetadata

@typing.final
class MearecWaveformStreamRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WAVEFORMTYPE_FIELD_NUMBER: builtins.int
    waveformType: global___WaveformType.ValueType
    def __init__(
        self,
        *,
        waveformType: global___WaveformType.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["waveformType", b"waveformType"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["waveformType", b"waveformType"]) -> None: ...

global___MearecWaveformStreamRequest = MearecWaveformStreamRequest

@typing.final
class MearecFileMeta(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROBEBASENAME_FIELD_NUMBER: builtins.int
    HSTGBASENAME_FIELD_NUMBER: builtins.int
    PATH_FIELD_NUMBER: builtins.int
    BASENAME_FIELD_NUMBER: builtins.int
    NUMTEMPLATES_FIELD_NUMBER: builtins.int
    WAVEFORMSHAPE_FIELD_NUMBER: builtins.int
    TEMPLATESHAPE_FIELD_NUMBER: builtins.int
    SAMPFREQ_FIELD_NUMBER: builtins.int
    TEMPLATESAMPFREQ_FIELD_NUMBER: builtins.int
    TR_FIELD_NUMBER: builtins.int
    HSTGDIR_FIELD_NUMBER: builtins.int
    PROBEDIR_FIELD_NUMBER: builtins.int
    WALLTIME_FIELD_NUMBER: builtins.int
    probeBaseName: builtins.str
    hstgBaseName: builtins.str
    path: builtins.str
    baseName: builtins.str
    numTemplates: builtins.int
    sampFreq: builtins.float
    templateSampFreq: builtins.float
    hstgDir: builtins.str
    probeDir: builtins.str
    wallTime: builtins.int
    @property
    def waveformShape(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def templateShape(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def TR(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        probeBaseName: builtins.str | None = ...,
        hstgBaseName: builtins.str | None = ...,
        path: builtins.str | None = ...,
        baseName: builtins.str | None = ...,
        numTemplates: builtins.int | None = ...,
        waveformShape: collections.abc.Iterable[builtins.int] | None = ...,
        templateShape: collections.abc.Iterable[builtins.int] | None = ...,
        sampFreq: builtins.float | None = ...,
        templateSampFreq: builtins.float | None = ...,
        TR: collections.abc.Iterable[builtins.float] | None = ...,
        hstgDir: builtins.str | None = ...,
        probeDir: builtins.str | None = ...,
        wallTime: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["baseName", b"baseName", "hstgBaseName", b"hstgBaseName", "hstgDir", b"hstgDir", "numTemplates", b"numTemplates", "path", b"path", "probeBaseName", b"probeBaseName", "probeDir", b"probeDir", "sampFreq", b"sampFreq", "templateSampFreq", b"templateSampFreq", "wallTime", b"wallTime"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["TR", b"TR", "baseName", b"baseName", "hstgBaseName", b"hstgBaseName", "hstgDir", b"hstgDir", "numTemplates", b"numTemplates", "path", b"path", "probeBaseName", b"probeBaseName", "probeDir", b"probeDir", "sampFreq", b"sampFreq", "templateSampFreq", b"templateSampFreq", "templateShape", b"templateShape", "wallTime", b"wallTime", "waveformShape", b"waveformShape"]) -> None: ...

global___MearecFileMeta = MearecFileMeta

@typing.final
class MearecWaveformChunk(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CHUNKIDX_FIELD_NUMBER: builtins.int
    WAVEFORMS_FIELD_NUMBER: builtins.int
    LASTCHUNK_FIELD_NUMBER: builtins.int
    chunkIdx: builtins.int
    lastChunk: builtins.bool
    @property
    def waveforms(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___MearecWaveform]: ...
    def __init__(
        self,
        *,
        chunkIdx: builtins.int | None = ...,
        waveforms: collections.abc.Iterable[global___MearecWaveform] | None = ...,
        lastChunk: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["chunkIdx", b"chunkIdx", "lastChunk", b"lastChunk"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["chunkIdx", b"chunkIdx", "lastChunk", b"lastChunk", "waveforms", b"waveforms"]) -> None: ...

global___MearecWaveformChunk = MearecWaveformChunk

@typing.final
class MearecTemplate(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    TEMPLATEIDX_FIELD_NUMBER: builtins.int
    PEAKNTVIDX_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    templateIdx: builtins.int
    peakNtvIdx: builtins.int
    def __init__(
        self,
        *,
        data: builtins.bytes | None = ...,
        templateIdx: builtins.int | None = ...,
        peakNtvIdx: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["data", b"data", "peakNtvIdx", b"peakNtvIdx", "templateIdx", b"templateIdx"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["data", b"data", "peakNtvIdx", b"peakNtvIdx", "templateIdx", b"templateIdx"]) -> None: ...

global___MearecTemplate = MearecTemplate

@typing.final
class MearecWaveform(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    SPIKETIME_FIELD_NUMBER: builtins.int
    TEMPLATEIDX_FIELD_NUMBER: builtins.int
    PEAKNTVIDX_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    spikeTime: builtins.float
    templateIdx: builtins.int
    peakNtvIdx: builtins.int
    def __init__(
        self,
        *,
        data: builtins.bytes | None = ...,
        spikeTime: builtins.float | None = ...,
        templateIdx: builtins.int | None = ...,
        peakNtvIdx: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["data", b"data", "peakNtvIdx", b"peakNtvIdx", "spikeTime", b"spikeTime", "templateIdx", b"templateIdx"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["data", b"data", "peakNtvIdx", b"peakNtvIdx", "spikeTime", b"spikeTime", "templateIdx", b"templateIdx"]) -> None: ...

global___MearecWaveform = MearecWaveform



"""Converters for impedance measurements."""

from __future__ import annotations

from typing import TYPE_CHECKING, cast

from radiens_core._generated import common_pb2, datasource_pb2
from radiens_core.models.impedance import Impedance, ImpedanceValue

if TYPE_CHECKING:
    from radiens_core._generated.datasource_pb2 import Impedance as ProtoImpedance
    from radiens_core.models.ports import Port


def port_to_proto(port: Port) -> common_pb2.Port.ValueType:
    """Convert domain Port to proto Port value."""
    # models.ports.Port is an IntEnum matching proto values
    return cast("common_pb2.Port.ValueType", int(port))


def impedance_from_proto(proto: ProtoImpedance) -> Impedance:
    """Convert proto Impedance to domain Impedance model."""
    return Impedance(
        measurements=[
            ImpedanceValue(
                ntv_chan_idx=val.ntvChanIdx,
                magnitude_ohm=val.zMag,
                phase_deg=val.zPhase,
            )
            for val in proto.impedances
        ]
    )


def impedance_request_to_proto(port: Port) -> datasource_pb2.ImpedanceRequest:
    """Create proto ImpedanceRequest from domain Port."""
    return datasource_pb2.ImpedanceRequest(port=port_to_proto(port))



"""Tests for enum converters — roundtrip domain -> proto -> domain."""

from radiens_core._converters._channels import signal_type_from_proto, signal_type_to_proto
from radiens_core._converters._common_enums import (
    backbone_mode_from_proto,
    backbone_mode_to_proto,
    port_from_proto,
    port_to_proto,
    record_mode_from_proto,
    record_mode_to_proto,
    stream_mode_from_proto,
    stream_mode_to_proto,
)
from radiens_core._converters._dataset import file_type_from_proto, file_type_to_proto
from radiens_core._converters._time_range import trs_mode_from_proto, trs_mode_to_proto
from radiens_core.models.dataset import RadiensFileType, TrsMode
from radiens_core.models.ports import Port
from radiens_core.models.signals import SignalType
from radiens_core.models.status import BackboneMode, RecordMode, StreamMode


def test_signal_type_roundtrip() -> None:
    for st in SignalType:
        proto_val = signal_type_to_proto(st)
        back = signal_type_from_proto(int(proto_val))
        assert back == st, f"SignalType roundtrip failed for {st}"


def test_trs_mode_roundtrip() -> None:
    for mode in TrsMode:
        proto_val = trs_mode_to_proto(mode)
        back = trs_mode_from_proto(int(proto_val))
        assert back == mode, f"TrsMode roundtrip failed for {mode}"


def test_file_type_roundtrip() -> None:
    for ft in RadiensFileType:
        proto_val = file_type_to_proto(ft)
        back = file_type_from_proto(proto_val)
        assert back == ft, f"RadiensFileType roundtrip failed for {ft}"


def test_stream_mode_roundtrip() -> None:
    for mode in StreamMode:
        proto_val = stream_mode_to_proto(mode)
        back = stream_mode_from_proto(int(proto_val))
        assert back == mode, f"StreamMode roundtrip failed for {mode}"


def test_record_mode_roundtrip() -> None:
    for mode in RecordMode:
        proto_val = record_mode_to_proto(mode)
        back = record_mode_from_proto(int(proto_val))
        assert back == mode, f"RecordMode roundtrip failed for {mode}"


def test_port_roundtrip() -> None:
    for port in Port:
        proto_val = port_to_proto(port)
        back = port_from_proto(int(proto_val))
        assert back == port, f"Port roundtrip failed for {port}"


def test_backbone_mode_roundtrip() -> None:
    for mode in BackboneMode:
        proto_val = backbone_mode_to_proto(mode)
        back = backbone_mode_from_proto(int(proto_val))
        assert back == mode, f"BackboneMode roundtrip failed for {mode}"



"""Tests for AllegoClient healthcheck method."""

from __future__ import annotations

from unittest.mock import MagicMock, patch

import pytest

from radiens_core import AllegoClient
from radiens_core._generated import common_pb2
from radiens_core.exceptions import RadiensError
from radiens_core.models.common import ServiceEndpoint

_FAKE_ADDRESSES: dict[str, str] = {
    ServiceEndpoint.ALLEGO_CORE: "localhost:50051",
    ServiceEndpoint.ALLEGO_LIFECYCLE: "localhost:50051",
    ServiceEndpoint.ALLEGO_PCACHE: "localhost:50052",
}


def _make_client() -> AllegoClient:
    """Create an AllegoClient with discovery mocked out."""
    with patch.object(AllegoClient, "_discover", return_value=dict(_FAKE_ADDRESSES)):
        return AllegoClient()


class TestHealthcheck:
    def test_healthcheck_success(self) -> None:
        """Test successful healthcheck call."""
        mock_pool = MagicMock()
        mock_channel = MagicMock()
        mock_pool.get_channel.return_value = mock_channel

        # Mock successful response
        expected_reply = common_pb2.StandardReply()
        expected_reply.errMsg = ""  # Empty error message indicates success

        with patch("radiens_core._services._allego_core.allegoserver_pb2_grpc.AllegoLifecycleStub") as mock_stub_cls:
            mock_stub = MagicMock()
            mock_stub.Healthcheck.return_value = expected_reply
            mock_stub_cls.return_value = mock_stub

            client = _make_client()
            client._pool = mock_pool  # Inject mock pool
            # Should not raise any exception
            client.healthcheck()

        mock_pool.get_channel.assert_called_once_with(client._core_address)
        mock_stub_cls.assert_called_once_with(mock_channel)
        mock_stub.Healthcheck.assert_called_once()

    def test_healthcheck_failure(self) -> None:
        """Test healthcheck call with server error."""
        mock_pool = MagicMock()
        mock_channel = MagicMock()
        mock_pool.get_channel.return_value = mock_channel

        # Mock error response
        expected_reply = common_pb2.StandardReply()
        expected_reply.errMsg = "Connection timeout"

        with patch("radiens_core._services._allego_core.allegoserver_pb2_grpc.AllegoLifecycleStub") as mock_stub_cls:
            mock_stub = MagicMock()
            mock_stub.Healthcheck.return_value = expected_reply
            mock_stub_cls.return_value = mock_stub

            client = _make_client()
            client._pool = mock_pool  # Inject mock pool
            with pytest.raises(RadiensError, match="Health check failed: Connection timeout"):
                client.healthcheck()



"""Typing helpers for post-validation narrowing."""

from __future__ import annotations

from typing import TypeVar

_T = TypeVar("_T")


def validated[T](v: object, expect: type[T]) -> T:
    """Narrow a ``@validate_call``-coerced value to its concrete type.

    Replaces ``cast()`` with a real runtime ``isinstance`` check
    so mypy narrows the type and incorrect usage fails fast.

    Usage::

        @validate_call(config=ConfigDict(arbitrary_types_allowed=True))
        def get_signals(self, time_range: FlexTimeRange, ...) -> ...:
            tr_spec = validated(time_range, TimeRangeSpec)
            # mypy knows tr_spec is TimeRangeSpec
    """
    assert isinstance(v, expect), f"Expected {expect.__name__} after @validate_call, got {type(v).__name__}"
    return v



"""
Integration test for new stimulation control methods in AllegoClient.

This test verifies that:
1. All new client methods exist and have correct signatures
2. Request/response models work correctly
3. Enum conversions work as expected
"""

from __future__ import annotations

import pytest
from pydantic import ValidationError

from radiens_core.allego_client import AllegoClient
from radiens_core.models.ports import Port
from radiens_core.models.stim import StimKeypressIndex, StimStep, StimStepRequest, StimStepState
from radiens_core.models.triggers import (
    ManualStimTriggerRequest,
    ManualStimTriggerToggleRequest,
    SetTriggerStateRequest,
    TriggerChannel,
    TriggerState,
)


def test_stimulation_control_methods_exist() -> None:
    """Test that all new stimulation control methods exist on AllegoClient."""
    from unittest.mock import patch

    with patch.object(
        AllegoClient,
        "_discover",
        return_value={
            "ALLEGO_CORE": "localhost:50051",
            "ALLEGO_LIFECYCLE": "localhost:50051",
            "ALLEGO_PCACHE": "localhost:50052",
        },
    ):
        client = AllegoClient()

    # Verify method existence
    assert hasattr(client, "set_trigger_state")
    assert hasattr(client, "get_trigger_state")
    assert hasattr(client, "manual_stim_trigger")
    assert hasattr(client, "manual_stim_trigger_toggle")
    assert hasattr(client, "set_stim_step")
    assert hasattr(client, "get_stim_step")


def test_trigger_state_models() -> None:
    """Test that trigger state models work correctly."""
    # Test SetTriggerStateRequest
    request = SetTriggerStateRequest(channel=TriggerChannel.T_DIN_0, enabled=True, port=Port.A)
    assert request.channel == TriggerChannel.T_DIN_0
    assert request.enabled is True
    assert request.port == Port.A

    # Test TriggerState response
    state = TriggerState(enabled_channels=[TriggerChannel.T_DIN_0, TriggerChannel.T_DIN_1], ports=[Port.A, Port.B])
    assert len(state.enabled_channels) == 2
    assert len(state.ports) == 2


def test_manual_stim_trigger_models() -> None:
    """Test manual stim trigger models."""
    # Test ManualStimTriggerRequest
    request = ManualStimTriggerRequest(trigger=StimKeypressIndex.KEYPRESS_1, trigger_on=True)
    assert request.trigger == StimKeypressIndex.KEYPRESS_1
    assert request.trigger_on is True

    # Test ManualStimTriggerToggleRequest
    toggle_request = ManualStimTriggerToggleRequest(trigger=StimKeypressIndex.KEYPRESS_1)
    assert toggle_request.trigger == StimKeypressIndex.KEYPRESS_1


def test_stim_step_models() -> None:
    """Test stimulation step models."""
    # Test StimStepRequest
    request = StimStepRequest(stim_step=StimStep.STIM_STEP_SIZE_100_NA)
    assert request.stim_step == StimStep.STIM_STEP_SIZE_100_NA

    # Test StimStepState
    state = StimStepState(stim_step=StimStep.STIM_STEP_SIZE_1_UA)
    assert state.stim_step == StimStep.STIM_STEP_SIZE_1_UA


def test_trigger_channel_enum_coverage() -> None:
    """Test that TriggerChannel enum has expected values."""
    # Test DIN channels
    assert hasattr(TriggerChannel, "T_DIN_0")
    assert hasattr(TriggerChannel, "T_DIN_15")

    # Test DOUT channels
    assert hasattr(TriggerChannel, "T_DOUT_0")
    assert hasattr(TriggerChannel, "T_DOUT_15")

    # Test special values
    assert hasattr(TriggerChannel, "T_NONE")

    # Ensure we have the right number of values (32 DIN/DOUT + 1 NONE)
    trigger_channels = list(TriggerChannel)
    assert len(trigger_channels) == 33


def test_stim_step_enum_coverage() -> None:
    """Test that StimStep enum has expected values."""
    # Test key step sizes
    assert hasattr(StimStep, "STIM_STEP_SIZE_MIN")
    assert hasattr(StimStep, "STIM_STEP_SIZE_10_NA")
    assert hasattr(StimStep, "STIM_STEP_SIZE_20_NA")
    assert hasattr(StimStep, "STIM_STEP_SIZE_50_NA")
    assert hasattr(StimStep, "STIM_STEP_SIZE_100_NA")
    assert hasattr(StimStep, "STIM_STEP_SIZE_200_NA")
    assert hasattr(StimStep, "STIM_STEP_SIZE_500_NA")
    assert hasattr(StimStep, "STIM_STEP_SIZE_1_UA")
    assert hasattr(StimStep, "STIM_STEP_SIZE_2_UA")
    assert hasattr(StimStep, "STIM_STEP_SIZE_5_UA")
    assert hasattr(StimStep, "STIM_STEP_SIZE_10_UA")
    assert hasattr(StimStep, "STIM_STEP_SIZE_MAX")

    # Ensure we have all expected step sizes
    step_sizes = list(StimStep)
    assert len(step_sizes) == 12


def test_model_immutability() -> None:
    """Test that all stimulation control models are immutable (frozen)."""
    # Test that models cannot be modified after creation
    request = SetTriggerStateRequest(channel=TriggerChannel.T_DIN_0, enabled=True, port=Port.A)

    with pytest.raises(ValidationError):
        request.channel = TriggerChannel.T_DIN_1  # type: ignore[misc]  # Should fail - model is frozen

    state = StimStepState(stim_step=StimStep.STIM_STEP_SIZE_1_UA)
    with pytest.raises(ValidationError):
        state.stim_step = StimStep.STIM_STEP_SIZE_10_UA  # type: ignore[misc]  # Should fail - model is frozen


if __name__ == "__main__":
    # Run tests directly
    pytest.main([__file__, "-v"])



"""KPI domain model <-> proto converters."""

from __future__ import annotations

from typing import cast

import numpy as np
import numpy.typing as npt

from radiens_core._generated import common_pb2
from radiens_core.models.kpi import KpiMetric, KpiMetricId, KpiMetricsResult, KpiMode, KpiStatus
from radiens_core.models.signals import SignalType
from radiens_core.models.time_range import TimeRange

# --- KpiMode ---

_KPI_MODE_TO_PROTO: dict[KpiMode, common_pb2.KpiMetricsMode.ValueType] = {
    KpiMode.BASE: common_pb2.Kpi_MetricsMode_Base,
    KpiMode.AVG: common_pb2.Kpi_MetricsMode_Avg,
    KpiMode.STREAM: common_pb2.Kpi_MetricsMode_Stream,
}


def kpi_mode_to_proto(mode: KpiMode) -> common_pb2.KpiMetricsMode.ValueType:
    """Convert domain KpiMode to proto KpiMetricsMode."""
    return _KPI_MODE_TO_PROTO[mode]


# --- KpiMetric ---


def kpi_metric_to_proto(metric: KpiMetric) -> common_pb2.KpiMetricsEnum.ValueType:
    """Convert domain KpiMetric to proto KpiMetricsEnum."""
    return cast("common_pb2.KpiMetricsEnum.ValueType", metric.value)


# --- KpiMetricId ---


def kpi_metric_id_to_proto(mid: KpiMetricId) -> common_pb2.KpiMetricID:
    """Convert domain KpiMetricId to proto KpiMetricID."""
    return common_pb2.KpiMetricID(
        mode=kpi_mode_to_proto(mid.mode),
        name=kpi_metric_to_proto(mid.name),
    )


def kpi_metric_id_from_proto(pb: common_pb2.KpiMetricID) -> KpiMetricId:
    """Convert proto KpiMetricID to domain KpiMetricId."""
    return KpiMetricId(mode=KpiMode(pb.mode), name=KpiMetric(pb.name))


# --- SignalType (internal helpers) ---

_SIGNAL_TYPE_TO_PROTO: dict[SignalType, common_pb2.SignalType.ValueType] = {
    SignalType.AMP: common_pb2.PRI,
    SignalType.AIN: common_pb2.AUX,
    SignalType.DIN: common_pb2.DIN,
    SignalType.DOUT: common_pb2.DOUT,
}

_SIGNAL_TYPE_FROM_PROTO: dict[int, SignalType] = {
    int(common_pb2.PRI): SignalType.AMP,
    int(common_pb2.AUX): SignalType.AIN,
    int(common_pb2.DIN): SignalType.DIN,
    int(common_pb2.DOUT): SignalType.DOUT,
}


def _signal_type_to_proto(sig_type: SignalType) -> common_pb2.SignalType.ValueType:
    return _SIGNAL_TYPE_TO_PROTO[sig_type]


def _signal_type_from_proto(pb: int) -> SignalType:
    return _SIGNAL_TYPE_FROM_PROTO[pb]


# --- DenseMatrix ---


def _dense_matrix_to_numpy(matrix: common_pb2.DenseMatrix) -> npt.NDArray[np.float64]:
    """Unpack a DenseMatrix proto to a float64 numpy array of shape (rows, cols)."""
    rows = matrix.rows
    cols = matrix.cols
    if rows == 0 or cols == 0:
        return np.empty((rows, cols), dtype=np.float64)
    return np.frombuffer(matrix.data, dtype=np.float64).reshape(rows, cols)


# --- TimeRange ---


def _time_range_from_kpi_proto(pb: common_pb2.TimeRange) -> TimeRange:
    """Build a domain TimeRange from a proto TimeRange as used in KPI responses."""
    sec_start = pb.timeRangeSec[0] if len(pb.timeRangeSec) >= 1 else 0.0
    sec_end = pb.timeRangeSec[1] if len(pb.timeRangeSec) >= 2 else sec_start
    ts_start = pb.timestamp[0] if len(pb.timestamp) >= 1 else 0
    ts_end = pb.timestamp[1] if len(pb.timestamp) >= 2 else ts_start
    fs = pb.fs if pb.HasField("fs") else 0.0
    walltime = pb.wallTimeStart if pb.HasField("wallTimeStart") else ""
    return TimeRange(
        sec=(sec_start, sec_end),
        timestamp=(ts_start, ts_end),
        fs=fs,
        dur_sec=sec_end - sec_start,
        walltime_str=walltime,
        n=ts_end - ts_start,
    )


# --- KpiMetricsResult ---


def kpi_metrics_result_from_proto(pb: common_pb2.SignalMetricsPacketReply) -> KpiMetricsResult:
    """Convert SignalMetricsPacketReply proto to domain KpiMetricsResult."""
    return KpiMetricsResult(
        channel_indices=tuple(pb.ntvIdxs),
        metrics=tuple(kpi_metric_id_from_proto(m) for m in pb.metrics),
        signal_type=_signal_type_from_proto(int(pb.sigType)),
        values=_dense_matrix_to_numpy(pb.metricVals),
        time_range=_time_range_from_kpi_proto(pb.tR),
    )


# --- KpiStatus ---


def kpi_status_from_proto(pb: common_pb2.KpiStatusReply) -> KpiStatus:
    """Convert KpiStatusReply proto to domain KpiStatus."""
    tr: TimeRange | None = None
    if pb.HasField("tR"):
        tr = _time_range_from_kpi_proto(pb.tR)
    num_sigs: int | None = pb.numSigs if pb.HasField("numSigs") else None
    return KpiStatus(
        stream_group_id=pb.streamGroupId,
        num_packets_memory=pb.numPacketsMemory,
        packet_dur=pb.packetDur,
        update_period=pb.updatePeriod,
        persistence=pb.persistence,
        beta=pb.beta,
        is_tracking_signal_cache=pb.isTrackingSignalCache,
        num_packets=pb.numPackets,
        memory_bytes=pb.memoryBytes,
        num_sigs=num_sigs,
        time_range=tr,
    )


# --- BundleReq builder ---


def build_bundle_req(
    channel_indices: list[int],
    time_range_sec: list[float],
    metrics: list[KpiMetricId],
) -> common_pb2.BundleReq:
    """Build a BundleReq proto from domain types.

    Parameters
    ----------
    channel_indices
        Native channel indices (ntvIdxs).
    time_range_sec
        Time range as ``[start_sec, end_sec]`` floats.
    metrics
        List of KPI metric identifiers.
    """
    return common_pb2.BundleReq(
        ntvIdxs=channel_indices,
        tR=common_pb2.TimeRange(timeRangeSec=time_range_sec),
        metrics=[kpi_metric_id_to_proto(m) for m in metrics],
    )



"""Tests for KPI converters and client methods."""

from __future__ import annotations

from typing import TYPE_CHECKING
from unittest.mock import patch

import numpy as np
import pytest

from radiens_core._constants import PRIMARY_CACHE_STREAM_GROUP_ID
from radiens_core._converters._kpi import (
    _dense_matrix_to_numpy,
    build_bundle_req,
    kpi_metric_id_from_proto,
    kpi_metric_id_to_proto,
    kpi_metrics_result_from_proto,
    kpi_mode_to_proto,
    kpi_status_from_proto,
)
from radiens_core._generated import common_pb2
from radiens_core.models.kpi import KpiMetric, KpiMetricId, KpiMetricsResult, KpiMode, KpiStatus
from radiens_core.models.signals import SignalType
from radiens_core.models.time_range import TimeRange

if TYPE_CHECKING:
    from radiens_core.models.dataset import DatasetMetadata

# =====================================================================
# Converter tests
# =====================================================================


def test_kpi_mode_to_proto_all_values() -> None:
    assert kpi_mode_to_proto(KpiMode.BASE) == common_pb2.Kpi_MetricsMode_Base
    assert kpi_mode_to_proto(KpiMode.AVG) == common_pb2.Kpi_MetricsMode_Avg
    assert kpi_mode_to_proto(KpiMode.STREAM) == common_pb2.Kpi_MetricsMode_Stream


def test_kpi_metric_id_roundtrip() -> None:
    mid = KpiMetricId(mode=KpiMode.BASE, name=KpiMetric.MEAN)
    proto = kpi_metric_id_to_proto(mid)
    assert proto.mode == common_pb2.Kpi_MetricsMode_Base
    assert proto.name == common_pb2.KPI_MEAN
    back = kpi_metric_id_from_proto(proto)
    assert back == mid


def test_kpi_metric_id_roundtrip_avg_rms() -> None:
    mid = KpiMetricId(mode=KpiMode.AVG, name=KpiMetric.RMS)
    back = kpi_metric_id_from_proto(kpi_metric_id_to_proto(mid))
    assert back == mid


def test_dense_matrix_to_numpy_normal() -> None:
    arr = np.array([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]], dtype=np.float64)
    matrix = common_pb2.DenseMatrix(rows=2, cols=3, data=arr.tobytes())
    result = _dense_matrix_to_numpy(matrix)
    np.testing.assert_array_equal(result, arr)
    assert result.shape == (2, 3)


def test_dense_matrix_to_numpy_empty_rows() -> None:
    matrix = common_pb2.DenseMatrix(rows=0, cols=3, data=b"")
    result = _dense_matrix_to_numpy(matrix)
    assert result.shape == (0, 3)


def test_dense_matrix_to_numpy_empty_cols() -> None:
    matrix = common_pb2.DenseMatrix(rows=4, cols=0, data=b"")
    result = _dense_matrix_to_numpy(matrix)
    assert result.shape == (4, 0)


def test_kpi_metrics_result_from_proto() -> None:
    vals = np.array([[1.5, 2.5], [3.5, 4.5]], dtype=np.float64)
    reply = common_pb2.SignalMetricsPacketReply(
        ntvIdxs=[0, 1],
        sigType=common_pb2.PRI,
        metrics=[
            common_pb2.KpiMetricID(mode=common_pb2.Kpi_MetricsMode_Base, name=common_pb2.KPI_MEAN),
            common_pb2.KpiMetricID(mode=common_pb2.Kpi_MetricsMode_Base, name=common_pb2.KPI_RMS),
        ],
        metricVals=common_pb2.DenseMatrix(rows=2, cols=2, data=vals.tobytes()),
        tR=common_pb2.TimeRange(
            timeRangeSec=[0.0, 1.0],
            timestamp=[0, 30000],
            fs=30000.0,
            wallTimeStart="2024-01-01 00:00:00",
        ),
    )
    result = kpi_metrics_result_from_proto(reply)
    assert isinstance(result, KpiMetricsResult)
    assert result.channel_indices == (0, 1)
    assert result.signal_type == SignalType.AMP
    assert len(result.metrics) == 2
    assert result.metrics[0].name == KpiMetric.MEAN
    assert result.metrics[1].name == KpiMetric.RMS
    np.testing.assert_array_equal(result.values, vals)
    assert result.time_range.sec == (0.0, 1.0)
    assert result.time_range.fs == 30000.0


def test_kpi_metrics_result_ain_signal_type() -> None:
    vals = np.array([[7.0]], dtype=np.float64)
    reply = common_pb2.SignalMetricsPacketReply(
        ntvIdxs=[2],
        sigType=common_pb2.AUX,
        metrics=[common_pb2.KpiMetricID(mode=common_pb2.Kpi_MetricsMode_Base, name=common_pb2.KPI_SNR)],
        metricVals=common_pb2.DenseMatrix(rows=1, cols=1, data=vals.tobytes()),
        tR=common_pb2.TimeRange(timeRangeSec=[0.0, 0.5], fs=30000.0),
    )
    result = kpi_metrics_result_from_proto(reply)
    assert result.signal_type == SignalType.AIN


def test_kpi_status_from_proto_no_optional_fields() -> None:
    pb = common_pb2.KpiStatusReply(
        streamGroupId="Live Signals",
        numPacketsMemory=10,
        packetDur=0.5,
        updatePeriod=0.1,
        persistence=0.9,
        beta=0.1,
        isTrackingSignalCache=True,
        numPackets=100,
        memoryBytes=1024.0,
    )
    status = kpi_status_from_proto(pb)
    assert isinstance(status, KpiStatus)
    assert status.stream_group_id == PRIMARY_CACHE_STREAM_GROUP_ID
    assert status.num_packets_memory == 10
    assert status.packet_dur == 0.5
    assert status.update_period == 0.1
    assert status.is_tracking_signal_cache is True
    assert status.num_packets == 100
    assert status.time_range is None
    assert status.num_sigs is None


def test_kpi_status_from_proto_with_optional_fields() -> None:
    pb = common_pb2.KpiStatusReply(
        streamGroupId="ds_001",
        numPacketsMemory=5,
        packetDur=1.0,
        updatePeriod=0.5,
        persistence=0.8,
        beta=0.2,
        isTrackingSignalCache=False,
        numPackets=50,
        memoryBytes=512.0,
        tR=common_pb2.TimeRange(timeRangeSec=[0.0, 50.0], fs=30000.0),
        numSigs=64,
    )
    status = kpi_status_from_proto(pb)
    assert status.num_sigs == 64
    assert status.time_range is not None
    assert status.time_range.sec == (0.0, 50.0)


def test_build_bundle_req() -> None:
    metrics = [KpiMetricId(mode=KpiMode.BASE, name=KpiMetric.MEAN)]
    req = build_bundle_req([0, 1, 2], [0.0, 5.0], metrics)
    assert list(req.ntvIdxs) == [0, 1, 2]
    assert list(req.tR.timeRangeSec) == [0.0, 5.0]
    assert len(req.metrics) == 1
    assert req.metrics[0].name == common_pb2.KPI_MEAN


# =====================================================================
# AllegoClient KPI tests
# =====================================================================


@pytest.fixture
def mock_allego_discovery() -> object:
    with patch("radiens_core.allego_client.AllegoClient._discover") as mock:
        mock.return_value = {
            "ALLEGO_CORE": "localhost:50051",
            "ALLEGO_PCACHE": "localhost:50052",
            "ALLEGO_KPI": "localhost:50053",
        }
        yield mock


@pytest.fixture
def mock_allego_auth() -> object:
    with patch("radiens_core._base_client.AuthInterceptor") as mock:
        yield mock


def test_allego_get_kpi_metrics_builds_correct_request(
    mock_allego_discovery: object,
    mock_allego_auth: object,
) -> None:
    from radiens_core.allego_client import AllegoClient

    client = AllegoClient()
    vals = np.array([[1.0, 2.0]], dtype=np.float64)
    mock_reply = common_pb2.SignalMetricsPacketReply(
        ntvIdxs=[0],
        sigType=common_pb2.PRI,
        metrics=[common_pb2.KpiMetricID(mode=common_pb2.Kpi_MetricsMode_Base, name=common_pb2.KPI_RMS)],
        metricVals=common_pb2.DenseMatrix(rows=1, cols=2, data=vals.tobytes()),
        tR=common_pb2.TimeRange(timeRangeSec=[0.0, 1.0], fs=30000.0),
    )

    with patch("radiens_core.allego_client.kpi_get_metrics_allego_proto", return_value=mock_reply) as mock_rpc:
        result = client.get_kpi_metrics(
            metrics=[KpiMetricId(mode=KpiMode.BASE, name=KpiMetric.RMS)],
            channel_indices=[0],
            time_range=[1.0, float("nan")],
        )

    assert isinstance(result, KpiMetricsResult)
    mock_rpc.assert_called_once()
    _pool, _addr, req = mock_rpc.call_args[0]
    assert req.streamGroupId == PRIMARY_CACHE_STREAM_GROUP_ID
    assert req.stype == common_pb2.PRI
    assert list(req.arg.ntvIdxs) == [0]
    assert req.arg.metrics[0].name == common_pb2.KPI_RMS


def test_allego_get_kpi_status(
    mock_allego_discovery: object,
    mock_allego_auth: object,
) -> None:
    from radiens_core.allego_client import AllegoClient

    client = AllegoClient()
    mock_reply = common_pb2.KpiStatusReply(
        streamGroupId=PRIMARY_CACHE_STREAM_GROUP_ID,
        numPacketsMemory=20,
        packetDur=0.5,
        updatePeriod=0.1,
        persistence=0.9,
        beta=0.1,
        isTrackingSignalCache=True,
        numPackets=200,
        memoryBytes=4096.0,
    )

    with patch("radiens_core.allego_client.get_kpi_status_allego_proto", return_value=mock_reply):
        status = client.get_kpi_status()

    assert isinstance(status, KpiStatus)
    assert status.num_packets_memory == 20


def test_allego_set_kpi_packet_dur_raises_on_error(
    mock_allego_discovery: object,
    mock_allego_auth: object,
) -> None:
    from radiens_core.allego_client import AllegoClient
    from radiens_core.exceptions import RadiensError

    client = AllegoClient()
    mock_reply = common_pb2.StandardReply(errMsg="server error")

    with (
        patch("radiens_core.allego_client.set_kpi_packet_dur_allego_proto", return_value=mock_reply),
        pytest.raises(RadiensError, match="Failed to set KPI packet duration"),
    ):
        client.set_kpi_packet_dur(0.5)


def test_allego_set_kpi_update_period_raises_on_error(
    mock_allego_discovery: object,
    mock_allego_auth: object,
) -> None:
    from radiens_core.allego_client import AllegoClient
    from radiens_core.exceptions import RadiensError

    client = AllegoClient()
    mock_reply = common_pb2.StandardReply(errMsg="bad param")

    with (
        patch("radiens_core.allego_client.set_kpi_update_period_proto", return_value=mock_reply),
        pytest.raises(RadiensError, match="Failed to set KPI update period"),
    ):
        client.set_kpi_update_period(0.1)


# =====================================================================
# VidereClient KPI tests
# =====================================================================


@pytest.fixture
def mock_videre_discovery() -> object:
    with patch("radiens_core._base_file_client.discover_radiens_addresses") as mock:
        mock.return_value = {
            "RADIENS_CORE": "localhost:50055",
            "RADIENS_LIFECYCLE": "localhost:50055",
        }
        yield mock


@pytest.fixture
def mock_videre_auth() -> object:
    with patch("radiens_core._base_client.AuthInterceptor") as mock:
        yield mock


def _make_videre_meta() -> DatasetMetadata:
    from radiens_core.models.channels import ChannelMetadata, KeyIdxs, SignalUnits
    from radiens_core.models.dataset import DatasetMetadata, RadiensFileType

    sig_map = {st: KeyIdxs(ntv=[0], dset=[0], sys=[0]) for st in SignalType}
    channel_metadata = ChannelMetadata(
        dataset_uid="uid_001",
        fs=30000.0,
        source_label="test",
        neighbor_max_radius_um=100.0,
        signal_units={SignalType.AMP: SignalUnits.MICROVOLTS},
        has_discrete=False,
        has_continuous_amp=True,
        sig_map=sig_map,
        selected_sig_map=sig_map,
        num_channels={st: 1 for st in SignalType},
        selected_num_channels={st: 1 for st in SignalType},
        site_positions=[],
    )
    time_range = TimeRange(
        sec=(0.0, 10.0),
        timestamp=(0, 300000),
        fs=30000.0,
        dur_sec=10.0,
        walltime_str="2024-01-01 00:00:00",
        n=300000,
    )
    return DatasetMetadata(
        id="ds_001",
        base_name="recording",
        path="/path/to/file.xdat",
        file_type=RadiensFileType.XDAT,
        time_range=time_range,
        channel_metadata=channel_metadata,
    )


def test_videre_get_kpi_metrics_uses_dataset_id(
    mock_videre_discovery: object,
    mock_videre_auth: object,
) -> None:
    from radiens_core.videre_client import VidereClient

    client = VidereClient()
    meta = _make_videre_meta()
    client._datasets[meta.id] = meta

    vals = np.array([[5.0]], dtype=np.float64)
    mock_reply = common_pb2.SignalMetricsPacketReply(
        ntvIdxs=[0],
        sigType=common_pb2.PRI,
        metrics=[common_pb2.KpiMetricID(mode=common_pb2.Kpi_MetricsMode_Base, name=common_pb2.KPI_MEAN)],
        metricVals=common_pb2.DenseMatrix(rows=1, cols=1, data=vals.tobytes()),
        tR=common_pb2.TimeRange(timeRangeSec=[0.0, 1.0], fs=30000.0),
    )

    with patch("radiens_core.videre_client.kpi_get_metrics_radiens_proto", return_value=mock_reply) as mock_rpc:
        result = client.get_kpi_metrics(
            dataset=meta,
            metrics=[KpiMetricId(mode=KpiMode.BASE, name=KpiMetric.MEAN)],
            channel_indices=[0],
            time_range=[0.0, 1.0],
        )

    assert isinstance(result, KpiMetricsResult)
    mock_rpc.assert_called_once()
    _pool, _addr, req = mock_rpc.call_args[0]
    assert req.streamGroupId == "ds_001"
    assert list(req.arg.ntvIdxs) == [0]


def test_videre_kpi_calculate_uses_dataset_id(
    mock_videre_discovery: object,
    mock_videre_auth: object,
) -> None:
    from radiens_core.videre_client import VidereClient

    client = VidereClient()
    meta = _make_videre_meta()
    client._datasets[meta.id] = meta

    with patch("radiens_core.videre_client.kpi_calculate_proto", return_value=common_pb2.StandardReply()) as mock_rpc:
        client.kpi_calculate(meta)

    mock_rpc.assert_called_once()
    _pool, _addr, req = mock_rpc.call_args[0]
    assert list(req.dsourceID) == ["ds_001"]


def test_videre_kpi_clear_uses_dataset_id(
    mock_videre_discovery: object,
    mock_videre_auth: object,
) -> None:
    from radiens_core.videre_client import VidereClient

    client = VidereClient()
    meta = _make_videre_meta()
    client._datasets[meta.id] = meta

    with patch("radiens_core.videre_client.kpi_clear_proto", return_value=common_pb2.StandardReply()) as mock_rpc:
        client.kpi_clear(meta)

    mock_rpc.assert_called_once()
    _pool, _addr, req = mock_rpc.call_args[0]
    assert list(req.dsourceID) == ["ds_001"]


def test_videre_set_kpi_packet_dur(
    mock_videre_discovery: object,
    mock_videre_auth: object,
) -> None:
    from radiens_core.videre_client import VidereClient

    client = VidereClient()
    meta = _make_videre_meta()
    client._datasets[meta.id] = meta

    with patch(
        "radiens_core.videre_client.set_kpi_packet_dur_radiens_proto",
        return_value=common_pb2.StandardReply(),
    ) as mock_rpc:
        client.set_kpi_packet_dur(meta, 2.0)

    mock_rpc.assert_called_once()
    _pool, _addr, req = mock_rpc.call_args[0]
    assert req.streamGroupId == "ds_001"
    assert req.param == 2.0


def test_videre_kpi_calculate_raises_on_error(
    mock_videre_discovery: object,
    mock_videre_auth: object,
) -> None:
    from radiens_core.exceptions import RadiensError
    from radiens_core.videre_client import VidereClient

    client = VidereClient()
    meta = _make_videre_meta()
    client._datasets[meta.id] = meta

    with (
        patch(
            "radiens_core.videre_client.kpi_calculate_proto",
            return_value=common_pb2.StandardReply(errMsg="compute failed"),
        ),
        pytest.raises(RadiensError, match="KPI calculate failed"),
    ):
        client.kpi_calculate(meta)



#!/usr/bin/env python3
"""Example usage of SetRecordState method."""

from time import sleep

from radiens_core import AllegoClient
from radiens_core.models.status import RecordMode, RecordStateRequest


def main() -> None:
    """Demonstrate SetRecordState usage."""
    with AllegoClient() as client:
        # System-level recording control (all ports)
        print("=== System-Level Recording ===")

        # Start recording (system level)
        start_request = RecordStateRequest(mode=RecordMode.ON)
        client.set_record_state(start_request)
        print("✅ System recording started")
        sleep(2)  # Simulate some recording time

        # Check status
        status = client.get_status()
        print(f"Current record mode: {status.recording.record_mode}")
        print(f"Active filename: {status.recording.active_filename}")
        sleep(2)  # Simulate more recording time

        # Stop recording (system level)
        stop_request = RecordStateRequest(mode=RecordMode.OFF)
        client.set_record_state(stop_request)
        print("✅ System recording stopped")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""Example usage of SetStreamState method."""

from time import sleep

from radiens_core import AllegoClient
from radiens_core.models.status import RecordMode, RecordStateRequest, StreamMode, StreamStateRequest


def main() -> None:
    """Demonstrate SetStreamState usage."""
    with AllegoClient() as client:
        # Turn streaming on
        start_request = StreamStateRequest(mode=StreamMode.ON)
        client.set_stream_state(start_request)

        sleep(5)  # Simulate some streaming time

        start_request = RecordStateRequest(mode=RecordMode.ON)
        client.set_record_state(start_request)

        # Check status
        status = client.get_status()
        print(f"Current stream mode: {status.streaming.stream_mode}")

        sleep(5)  # Simulate some streaming time

        stop_request = RecordStateRequest(mode=RecordMode.OFF)
        client.set_record_state(stop_request)

        sleep(5)

        # Turn streaming off
        stop_request = StreamStateRequest(mode=StreamMode.OFF)
        client.set_stream_state(stop_request)


if __name__ == "__main__":
    main()



"""Tests for _services/_allego_core.py — mock stub verification."""

from __future__ import annotations

from unittest.mock import MagicMock, patch

from radiens_core._generated import allegoserver_pb2, common_pb2
from radiens_core._services._allego_core import (
    get_stim_params_proto,
    healthcheck_proto,
    restart_proto,
    set_record_state_proto,
)


class TestGetStimParamsProto:
    def test_calls_stub_with_standard_request(self) -> None:
        """Verify the service calls GetStimParams with a StandardRequest."""
        mock_pool = MagicMock()
        mock_channel = MagicMock()
        mock_pool.get_channel.return_value = mock_channel

        expected_reply = allegoserver_pb2.StimParamsReply()

        with patch("radiens_core._services._allego_core.allegoserver_pb2_grpc.AllegoCoreStub") as mock_stub_cls:
            mock_stub = MagicMock()
            mock_stub.GetStimParams.return_value = expected_reply
            mock_stub_cls.return_value = mock_stub

            result = get_stim_params_proto(mock_pool, "localhost:50051")

        mock_pool.get_channel.assert_called_once_with("localhost:50051")
        mock_stub_cls.assert_called_once_with(mock_channel)
        mock_stub.GetStimParams.assert_called_once()

        # Verify the request arg is a StandardRequest
        call_args = mock_stub.GetStimParams.call_args
        request = call_args[0][0]
        assert isinstance(request, common_pb2.StandardRequest)

        assert result is expected_reply


class TestHealthcheckProto:
    def test_calls_stub_with_standard_request(self) -> None:
        """Verify the service calls Healthcheck with a StandardRequest."""
        mock_pool = MagicMock()
        mock_channel = MagicMock()
        mock_pool.get_channel.return_value = mock_channel

        expected_reply = common_pb2.StandardReply()

        with patch("radiens_core._services._allego_core.allegoserver_pb2_grpc.AllegoLifecycleStub") as mock_stub_cls:
            mock_stub = MagicMock()
            mock_stub.Healthcheck.return_value = expected_reply
            mock_stub_cls.return_value = mock_stub

            result = healthcheck_proto(mock_pool, "localhost:50051")

        mock_pool.get_channel.assert_called_once_with("localhost:50051")
        mock_stub_cls.assert_called_once_with(mock_channel)
        mock_stub.Healthcheck.assert_called_once()

        # Verify the request arg is a StandardRequest
        call_args = mock_stub.Healthcheck.call_args
        request = call_args[0][0]
        assert isinstance(request, common_pb2.StandardRequest)

        assert result is expected_reply


class TestSetRecordStateProto:
    def test_calls_stub_with_record_request(self) -> None:
        """Verify the service calls SetRecordState with a SetRecordRequest."""
        mock_pool = MagicMock()
        mock_channel = MagicMock()
        mock_pool.get_channel.return_value = mock_channel

        # Create a sample SetRecordRequest
        test_request = allegoserver_pb2.SetRecordRequest()
        test_request.mode = allegoserver_pb2.R_ON

        expected_reply = common_pb2.StandardReply()

        with patch("radiens_core._services._allego_core.allegoserver_pb2_grpc.AllegoCoreStub") as mock_stub_cls:
            mock_stub = MagicMock()
            mock_stub.SetRecordState.return_value = expected_reply
            mock_stub_cls.return_value = mock_stub

            result = set_record_state_proto(mock_pool, "localhost:50051", test_request)

        mock_pool.get_channel.assert_called_once_with("localhost:50051")
        mock_stub_cls.assert_called_once_with(mock_channel)
        mock_stub.SetRecordState.assert_called_once_with(test_request)

        assert result is expected_reply


class TestRestartProto:
    def test_calls_stub_with_restart_request(self) -> None:
        """Verify the service calls Restart with a RestartRequest."""
        mock_pool = MagicMock()
        mock_channel = MagicMock()
        mock_pool.get_channel.return_value = mock_channel

        # Create a sample RestartRequest
        test_request = allegoserver_pb2.RestartRequest()
        test_request.mode = allegoserver_pb2.SMARTBOX_PRO

        expected_reply = common_pb2.StandardReply()

        with patch("radiens_core._services._allego_core.allegoserver_pb2_grpc.AllegoCoreStub") as mock_stub_cls:
            mock_stub = MagicMock()
            mock_stub.Restart.return_value = expected_reply
            mock_stub_cls.return_value = mock_stub

            result = restart_proto(mock_pool, "localhost:50051", test_request)

        mock_pool.get_channel.assert_called_once_with("localhost:50051")
        mock_stub_cls.assert_called_once_with(mock_channel)
        mock_stub.Restart.assert_called_once_with(test_request)

        assert result is expected_reply



"""DAC (Analog Output) domain models."""

from __future__ import annotations

from pydantic import BaseModel, ConfigDict


class DACHighPass(BaseModel):
    """DAC high-pass filter configuration.

    Attributes:
        enable: Whether the high-pass filter is enabled.
        cutoff_freq: High-pass filter cutoff frequency in Hertz.
    """

    model_config = ConfigDict(frozen=True)

    enable: bool
    cutoff_freq: float


class AnalogOutChannel(BaseModel):
    """Configuration for a single analog output channel.

    Attributes:
        pri_ntv_chan_idx: Primary native channel index.
        analog_out_ntv_chan_idx: Analog output native channel index.
        stream: Stream identifier.
        stream_offset_idx: Index offset within the stream.
    """

    model_config = ConfigDict(frozen=True)

    pri_ntv_chan_idx: int
    analog_out_ntv_chan_idx: int
    stream: int
    stream_offset_idx: int


class AnalogOutRegister(BaseModel):
    """Complete DAC (Analog Output) register state.

    Attributes:
        gain: DAC gain setting.
        highpass_reg: High-pass filter configuration. References DACHighPass.
        channels: List of individual analog output channel configurations. References AnalogOutChannel.
    """

    model_config = ConfigDict(frozen=True)

    gain: int
    highpass_reg: DACHighPass
    channels: list[AnalogOutChannel]
